# !/usr/bin/python
# coding=utf-8
from uitk.slots.maya import *
from uitk.slots.edit import Edit


class Edit_maya(Edit, Slots_maya):
	def __init__(self, *args, **kwargs):
		super().__init__(*args, **kwargs)

		dh = self.sb.edit.draggable_header
		items = ['Cleanup', 'Transfer: Attribute Values', 'Transfer: Shading Sets']
		dh.ctxMenu.cmb000.addItems_(items, 'Maya Editors')

		tb000 = self.sb.edit.tb000
		tb000.ctxMenu.add('QCheckBox', setText='All Geometry', setObjectName='chk005', setToolTip='Clean All scene geometry.')
		tb000.ctxMenu.add('QCheckBox', setText='Repair', setObjectName='chk004', setToolTip='Repair matching geometry. Else, select only.') #add(self.sb.CheckBox, setText='Select Only', setObjectName='chk004', setTristate=True, setCheckState_=2, setToolTip='Select and/or Repair matching geometry. <br>0: Repair Only<br>1: Repair and Select<br>2: Select Only')
		tb000.ctxMenu.add('QCheckBox', setText='Merge vertices', setObjectName='chk024', setChecked=True, setToolTip='Merge overlapping vertices on the object(s) before executing the clean command.')
		tb000.ctxMenu.add('QCheckBox', setText='N-Gons', setObjectName='chk002', setChecked=True, setToolTip='Find N-gons.')
		tb000.ctxMenu.add('QCheckBox', setText='Non-Manifold Geometry', setObjectName='chk017', setChecked=True, setToolTip='Check for nonmanifold polys.')
		tb000.ctxMenu.add('QCheckBox', setText='Non-Manifold Vertex', setObjectName='chk021', setToolTip='A connected vertex of non-manifold geometry where the faces share a single vertex.')
		tb000.ctxMenu.add('QCheckBox', setText='Quads', setObjectName='chk010', setToolTip='Check for quad sided polys.')
		tb000.ctxMenu.add('QCheckBox', setText='Concave', setObjectName='chk011', setToolTip='Check for concave polys.')
		tb000.ctxMenu.add('QCheckBox', setText='Non-Planar', setObjectName='chk003', setToolTip='Check for non-planar polys.')
		tb000.ctxMenu.add('QCheckBox', setText='Holed', setObjectName='chk012', setToolTip='Check for holed polys.')
		tb000.ctxMenu.add('QCheckBox', setText='Lamina', setObjectName='chk018', setChecked=True, setToolTip='Check for lamina polys.')
		tb000.ctxMenu.add('QCheckBox', setText='Shared UV\'s', setObjectName='chk016', setToolTip='Unshare uvs that are shared across vertices.')
		# tb000.ctxMenu.add('QCheckBox', setText='Invalid Components', setObjectName='chk019', setToolTip='Check for invalid components.')
		tb000.ctxMenu.add('QCheckBox', setText='Zero Face Area', setObjectName='chk013', setChecked=True, setToolTip='Check for 0 area faces.')
		tb000.ctxMenu.add('QDoubleSpinBox', setPrefix='Face Area Tolerance:   ', setObjectName='s006', setMinMax_='0.0-10 step.000010', setValue=0.000010, setToolTip='Tolerance for face areas.')
		tb000.ctxMenu.add('QCheckBox', setText='Zero Length Edges', setObjectName='chk014', setChecked=True, setToolTip='Check for 0 length edges.')
		tb000.ctxMenu.add('QDoubleSpinBox', setPrefix='Edge Length Tolerance: ', setObjectName='s007', setMinMax_='0.0-10 step.000010', setValue=0.000010, setToolTip='Tolerance for edge length.')
		tb000.ctxMenu.add('QCheckBox', setText='Zero UV Face Area', setObjectName='chk015', setToolTip='Check for 0 uv face area.')
		tb000.ctxMenu.add('QDoubleSpinBox', setPrefix='UV Face Area Tolerance:', setObjectName='s008', setDisabled=True, setMinMax_='0.0-10 step.000010', setValue=0.000010, setToolTip='Tolerance for uv face areas.')
		tb000.ctxMenu.add('QCheckBox', setText='Overlapping Faces', setObjectName='chk025', setToolTip='Find any overlapping duplicate faces. (can be very slow on dense objects)')
		tb000.ctxMenu.add('QCheckBox', setText='Overlapping Duplicate Objects', setObjectName='chk022', setToolTip='Find any duplicate overlapping geometry at the object level.')
		tb000.ctxMenu.add('QCheckBox', setText='Omit Selected Objects', setObjectName='chk023', setDisabled=True, setToolTip='Overlapping Duplicate Objects: Search for duplicates of any selected objects while omitting the initially selected objects.')
		tb000.ctxMenu.chk013.toggled.connect(lambda state: tb000.ctxMenu.s006.setEnabled(True if state else False))
		tb000.ctxMenu.chk014.toggled.connect(lambda state: tb000.ctxMenu.s007.setEnabled(True if state else False))
		tb000.ctxMenu.chk015.toggled.connect(lambda state: tb000.ctxMenu.s008.setEnabled(True if state else False))
		tb000.ctxMenu.chk022.stateChanged.connect(lambda state: self.sb.toggleWidgets(tb000.ctxMenu, setDisabled='chk002-3,chk005,chk010-21,chk024,s006-8', setEnabled='chk023') if state 
														else self.sb.toggleWidgets(tb000.ctxMenu, setEnabled='chk002-3,chk005,chk010-21,s006-8', setDisabled='chk023')) #disable non-relevant options.
		#sync widgets
		self.sb.setSyncConnections(tb000.ctxMenu.chk004, self.sb.edit_submenu.chk004, attributes='setChecked')
		self.sb.setSyncConnections(tb000.ctxMenu.chk010, self.sb.edit_submenu.chk010, attributes='setChecked')


	def cmb000(self, index=-1):
		'''Editors
		'''
		cmb = self.sb.edit.draggable_header.ctxMenu.cmb000

		if index>0:
			text = cmb.items[index]
			if text=='Cleanup':
				pm.mel.CleanupPolygonOptions()
			elif text=='Transfer: Attribute Values':
				pm.mel.TransferAttributeValuesOptions()
				# pm.mel.eval('performTransferAttributes 1;') #Transfer Attributes Options
			elif text=='Transfer: Shading Sets':
				pm.mel.performTransferShadingSets(1)
			cmb.setCurrentIndex(0)


	@Slots_maya.attr
	def cmb001(self, index=-1):
		'''Object History Attributes
		'''
		cmb = self.sb.edit.cmb001

		try:
			items = list(set([n.name() for n in pm.listHistory(pm.ls(sl=1, objectsOnly=1), pruneDagObjects=1)])) #levels=1, interestLevel=2, 
		except RuntimeError as error:
			items = ['No selection.']
		cmb.addItems_(items, 'History')

		cmb.setCurrentIndex(0)
		if index>0:
			if cmb.items[index]!='No selection.':
				return pm.ls(cmb.items[index])


	def tb000(self, state=None):
		'''Mesh Cleanup
		'''
		tb = self.sb.edit.tb000

		allMeshes = int(tb.ctxMenu.chk005.isChecked()) #[0] All selectable meshes
		repair = tb.ctxMenu.chk004.isChecked() #repair or select only
		quads = int(tb.ctxMenu.chk010.isChecked()) #[3] check for quads polys
		mergeVertices = tb.ctxMenu.chk024.isChecked()
		nsided = int(tb.ctxMenu.chk002.isChecked()) #[4] check for n-sided polys
		concave = int(tb.ctxMenu.chk011.isChecked()) #[5] check for concave polys
		holed = int(tb.ctxMenu.chk012.isChecked()) #[6] check for holed polys
		nonplanar = int(tb.ctxMenu.chk003.isChecked()) #[7] check for non-planar polys
		zeroGeom = int(tb.ctxMenu.chk013.isChecked()) #[8] check for 0 area faces
		zeroGeomTol = tb.ctxMenu.s006.value() #[9] tolerance for face areas
		zeroEdge = int(tb.ctxMenu.chk014.isChecked()) #[10] check for 0 length edges
		zeroEdgeTol = tb.ctxMenu.s007.value() #[11] tolerance for edge length
		zeroMap = int(tb.ctxMenu.chk015.isChecked()) #[12] check for 0 uv face area
		zeroMapTol = tb.ctxMenu.s008.value() #[13] tolerance for uv face areas
		sharedUVs = int(tb.ctxMenu.chk016.isChecked()) #[14] Unshare uvs that are shared across vertices
		nonmanifold = int(tb.ctxMenu.chk017.isChecked()) #[15] check for nonmanifold polys
		lamina = -int(tb.ctxMenu.chk018.isChecked()) #[16] check for lamina polys [default -1]
		splitNonManifoldVertex = tb.ctxMenu.chk021.isChecked()
		invalidComponents = 0 #int(tb.ctxMenu.chk019.isChecked()) #[17] a guess what this arg does. not checked. default is 0.
		overlappingFaces = tb.ctxMenu.chk025.isChecked()
		overlappingDuplicateObjects = tb.ctxMenu.chk022.isChecked() #find overlapping geometry at object level.
		omitSelectedObjects = tb.ctxMenu.chk023.isChecked() #Search for duplicates of any selected objects while omitting the initially selected objects.

		objects = pm.ls(sl=1, transforms=1)

		if overlappingDuplicateObjects:
			duplicates = mtk.Edit.getOverlappingDupObjects(omitInitialObjects=omitSelectedObjects, select=True, verbose=True)
			self.messageBox('Found {} duplicate overlapping objects.'.format(len(duplicates)), messageType='Result')
			pm.delete(duplicates) if repair else pm.select(duplicates)
			return

		if mergeVertices:
			[pm.polyMergeVertex(obj.verts, distance=0.0001) for obj in objects] #merge vertices on each object.

		if overlappingFaces:
			duplicates = mtk.Edit.getOverlappingFaces(objects)
			self.messageBox('Found {} duplicate overlapping faces.'.format(len(duplicates)), messageType='Result')
			pm.delete(duplicates) if repair else pm.select(duplicates, add=1)

		mtk.Edit.cleanGeometry(objects, allMeshes=allMeshes, repair=repair, quads=quads, nsided=nsided, concave=concave, holed=holed, nonplanar=nonplanar, 
			zeroGeom=zeroGeom, zeroGeomTol=zeroGeomTol, zeroEdge=zeroEdge, zeroEdgeTol=zeroEdgeTol, zeroMap=zeroMap, zeroMapTol=zeroMapTol, 
			sharedUVs=sharedUVs, nonmanifold=nonmanifold, invalidComponents=invalidComponents, splitNonManifoldVertex=splitNonManifoldVertex)


	def tb001(self, state=None):
		'''Delete History
		'''
		tb = self.sb.edit.tb001

		all_ = tb.ctxMenu.chk018.isChecked()
		unusedNodes = tb.ctxMenu.chk019.isChecked()
		deformers = tb.ctxMenu.chk020.isChecked()
		optimize = tb.ctxMenu.chk030.isChecked()

		objects = pm.ls(selection=1, objectsOnly=1) if not all_ else pm.ls(typ="mesh")

		if unusedNodes:
			pm.mel.MLdeleteUnused() #pm.mel.hyperShadePanelMenuCommand('hyperShadePanel1', 'deleteUnusedNodes')
			#delete empty groups:
			empty = mtk.Node.getGroups(empty=True)
			pm.delete(empty)

		try: #delete history
			if all_:
				pm.delete(objects, constructionHistory=1)
			else:
				pm.bakePartialHistory(objects, prePostDeformers=1)
		except:
			pass

		if optimize:
			pm.mel.OptimizeScene()

		#display viewPort messages
		if all_:
			if deformers:
				mtk.viewportMessage("delete <hl>all</hl> history.")
			else:
				mtk.viewportMessage("delete <hl>all non-deformer</hl> history.")
		else:
			if deformers:
				mtk.viewportMessage("delete history on "+str(objects))
			else:
				mtk.viewportMessage("delete <hl>non-deformer</hl> history on "+str(objects))


	def tb002(self, state=None):
		'''Delete
		'''
		tb = self.sb.edit.tb002

		deleteRing = tb.ctxMenu.chk000.isChecked()
		deleteLoop = tb.ctxMenu.chk001.isChecked()

		# selectionMask = pm.selectMode (query=True, component=True)
		maskVertex = pm.selectType (query=True, vertex=True)
		maskEdge = pm.selectType (query=True, edge=True)
		maskFacet = pm.selectType (query=True, facet=True)

		objects = pm.ls(sl=1, objectsOnly=1)
		for obj in objects:
			if pm.objectType(obj, isType='joint'):
				pm.removeJoint(obj) #remove joints

			elif pm.objectType(obj, isType='mesh'): 
				if maskEdge:
					selection = pm.ls(obj, sl=1, flatten=1)
					if deleteRing:
						pm.polyDelEdge(mtk.Cmpt.getEdgePath(selection, 'edgeRing'), cleanVertices=True) # pm.polySelect(edges, edgeRing=True) #select the edge ring.
					if deleteLoop:
						pm.polyDelEdge(mtk.Cmpt.getEdgePath(selection, 'edgeLoop'), cleanVertices=True) # pm.polySelect(edges, edgeLoop=True) #select the edge loop.
					else:
						pm.polyDelEdge(selection, cleanVertices=True) #delete edges

				elif maskVertex:
					pm.polyDelVertex() #try delete vertices
					if pm.ls(sl=1)==objects: #if nothing was deleted:
						pm.mel.eval('polySelectSp -loop;') #convert selection to edge loop
						pm.polyDelEdge(cleanVertices=True) #delete edges

				else: #all([selectionMask==1, maskFacet==1]):
					pm.delete(obj) #delete faces\mesh objects


	def tb003(self, state=None):
		'''Delete Along Axis
		'''
		tb = self.sb.edit.tb003

		axis = self.sb.getAxisFromCheckBoxes('chk006-9', tb.ctxMenu)

		pm.undoInfo(openChunk=1)
		objects = pm.ls(sl=1, objectsOnly=1)

		for obj in objects:
			mtk.Edit.deleteAlongAxis(obj, axis)
		pm.undoInfo(closeChunk=1)


	@mtk.undo
	def tb004(self, state=None):
		'''Delete Along Axis
		'''
		tb = self.sb.edit.tb004

		allNodes = tb.ctxMenu.chk026.isChecked()
		unlock = tb.ctxMenu.chk027.isChecked()

		# pm.undoInfo(openChunk=1)
		nodes = pm.ls() if allNodes else pm.ls(selection=1)
		for node in nodes:
			pm.lockNode(node, lock=not unlock)
		# pm.undoInfo(closeChunk=1)


	@Slots.hideMain
	def b001(self):
		'''Object History Attributes: get most recent node
		'''
		cmb = self.sb.edit.cmb001
		self.cmb001() #refresh the contents of the combobox.

		items = pm.ls(cmb.items[-1])
		if items:
			self.setAttributeWindow(items, checkableLabel=True)
		else:
			self.messageBox('Found no items to list the history for.')
			return


	def b021(self):
		'''Tranfer Maps
		'''
		pm.mel.performSurfaceSampling(1)


	def b022(self):
		'''Transfer Vertex Order
		'''
		pm.mel.TransferVertexOrder()


	def b023(self):
		'''Transfer Attribute Values
		'''
		pm.mel.TransferAttributeValues()


	def b027(self):
		'''Shading Sets
		'''
		pm.mel.performTransferShadingSets(0)

# --------------------------------------------------------------------------------------------









#module name
print (__name__)
# --------------------------------------------------------------------------------------------
# Notes
# --------------------------------------------------------------------------------------------
	# b008, b009, b011



# !/usr/bin/python
# coding=utf-8
from uitk.slots.max import *
from uitk.slots.transform import Transform


class Transform_max(Transform, Slots_max):
	def __init__(self, *args, **kwargs):
		super().__init__(*args, **kwargs)

		cmb000 = self.sb.transform.draggable_header.ctxMenu.cmb000
		items = ['']
		cmb000.addItems_(items, '')

		cmb001 = self.sb.transform.cmb001
		cmb001.ctxMenu.add('QRadioButton', setObjectName='chk017', setText='Standard', setChecked=True, setToolTip='')
		cmb001.ctxMenu.add('QRadioButton', setObjectName='chk018', setText='Body Shapes', setToolTip='')
		cmb001.ctxMenu.add('QRadioButton', setObjectName='chk019', setText='NURBS', setToolTip='')
		cmb001.ctxMenu.add('QRadioButton', setObjectName='chk020', setText='Point Cloud Shapes', setToolTip='')
		cmb001.ctxMenu.add(self.sb.Label, setObjectName='lbl000', setText='Disable All', setToolTip='Disable all constraints.')
		self.sb.connect('chk017-20', 'toggled', self.cmb001, cmb001.ctxMenu) #connect to this method on toggle

		cmb002 = self.sb.transform.cmb002
		items = ['Point to Point', '2 Points to 2 Points', '3 Points to 3 Points', 'Align Objects', 'Position Along Curve', 'Align Tool', 'Snap Together Tool']
		cmb002.addItems_(items, 'Align To')

		cmb003 = self.sb.transform.cmb003
		# moveValue = pm.manipMoveContext('Move', q=True, snapValue=True)
		# cmb003.menu_.s021.setValue(moveValue)
		# scaleValue = pm.manipScaleContext('Scale', q=True, snapValue=True)
		# cmb003.menu_.s022.setValue(scaleValue)
		# rotateValue = pm.manipRotateContext('Rotate', q=True, snapValue=True)
		# cmb003.menu_.s023.setValue(rotateValue)


	def cmb000(self, index=-1):
		'''Editors
		'''
		cmb = self.sb.transform.draggable_header.ctxMenu.cmb000

		if index>0:
			text = cmb.items[index]
			if text=='':
				pass
			cmb.setCurrentIndex(0)


	def cmb001(self, index=-1):
		'''Transform Contraints

		constrain along normals #checkbox option for edge amd surface constaints
		setXformConstraintAlongNormal false;
		'''
		cmb = self.sb.transform.cmb001

		cmb.menu_.clear()
		if cmb.ctxMenu.chk017.isChecked(): #Standard
			cmb.setItemText(0,'Standard') #set cetagory title in standard model/view
			items = ['Grid Points', 'Pivot', 'Perpendicular', 'Vertex', 'Edge/Segment', 'Face', 'Grid Lines', 'Bounding Box', 'Tangent', 'Endpoint', 'Midpoint', 'Center Face']
		if cmb.ctxMenu.chk018.isChecked(): #Body Shapes
			cmb.setItemText(0,'Body Shapes') #set category title in standard model/view
			items = ['Vertex_', 'Edge', 'Face_', 'End Edge', 'Edge Midpoint']
		if cmb.ctxMenu.chk019.isChecked(): #NURBS
			cmb.setItemText(0,'NURBS') #set category title in standard model/view
			items = ['CV', 'Curve Center', 'Curve Tangent', 'Curve End', 'Surface Normal', 'Point', 'Curve Normal', 'Curve Edge', 'Surface Center','Surface Edge']
		if cmb.ctxMenu.chk020.isChecked(): #Point Cloud Shapes
			cmb.setItemText(0,'Point Cloud Shapes') #set category title in standard model/view
			items = ['Point Cloud Vertex']

		widgets = [cmb.menu_.add('QCheckBox', setText=t) for t in items]

		for w in widgets:
			try:
				w.disconnect() #disconnect all previous connections.
			except TypeError:
				pass #if no connections are present; pass
			w.toggled.connect(lambda state, widget=w: self.chkxxx(state=state, widget=widget))


	def cmb002(self, index=-1):
		'''Align To
		'''
		cmb = self.sb.transform.cmb002

		if index>0:
			text = cmb.items[index]
			if text=='Point to Point':
				mel.eval('SnapPointToPointOptions;') #performSnapPtToPt 1; Select any type of point object or component.
			elif text=='2 Points to 2 Points':
				mel.eval('Snap2PointsTo2PointsOptions;') #performSnap2PtTo2Pt 1; Select any type of point object or component.
			elif text=='3 Points to 3 Points':
				mel.eval('Snap3PointsTo3PointsOptions;') #performSnap3PtTo3Pt 1; Select any type of point object or component.
			elif text=='Align Objects':
				mel.eval('performAlignObjects 1;') #Align the selected objects.
			elif text=='Position Along Curve':
				mel.eval('PositionAlongCurve;') #Position selected objects along a selected curve.
				# import maya.app.general.positionAlongCurve
				# maya.app.general.positionAlongCurve.positionAlongCurve()
			elif text=='Align Tool':
				mel.eval('SetAlignTool;') #setToolTo alignToolCtx; Align the selection to the last selected object.
			elif text=='Snap Together Tool':
				mel.eval('SetSnapTogetherToolOptions;') #setToolTo snapTogetherToolCtx; toolPropertyWindow;) Snap two objects together.


	def cmb003(self, index=-1):
		'''Transform Tool Snapping
		'''
		cmb = self.sb.transform.cmb003


	def s021(self, value=None):
		'''Transform Tool Snap Settings: Spinboxes
		'''
		pm.manipMoveContext('Move', edit=1, snapValue=value)
		pm.texMoveContext('texMoveContext', edit=1, snapValue=value) #uv move context


	def s022(self, value=None):
		'''Transform Tool Snap Settings: Spinboxes
		'''
		pm.manipScaleContext('Scale', edit=1, snapValue=value)
		pm.texScaleContext('texScaleContext', edit=1, snapValue=value) #uv scale context


	def s023(self, value=None):
		'''Transform Tool Snap Settings: Spinboxes
		'''
		pm.manipRotateContext('Rotate', edit=1, snapValue=value)
		pm.texRotateContext('texRotateContext', edit=1, snapValue=value) #uv rotate context


	def chkxxx(self, **kwargs):
		'''Transform Constraints: Constraint CheckBoxes
		'''
		try:
			self.setSnapState(kwargs['widget'].text(), kwargs['state'])
		except KeyError:
			pass


	def tb000(self, state=None):
		'''Drop To Grid
		'''
		tb = self.sb.transform.tb000

		align = tb.ctxMenu.cmb004.currentText()
		origin = tb.ctxMenu.chk014.isChecked()
		centerPivot = tb.ctxMenu.chk016.isChecked()

		objects = pm.ls(sl=1, objectsOnly=1)
		Slots_max.dropToGrid(objects, align, origin, centerPivot)
		pm.select(objects) #reselect the original selection.


	def tb001(self, state=None):
		'''Align Vertices

		Auto Align finds the axis with the largest variance, and set the axis checkboxes accordingly before performing a regular align.
		'''
		tb = self.sb.transform.tb001

		betweenTwoComponents = tb.ctxMenu.chk013.isChecked()
		autoAlign = tb.ctxMenu.chk010.isChecked()
		autoAlign2Axes = tb.ctxMenu.chk011.isChecked() #Auto Align: Two Axes

		selection = pm.ls(orderedSelection=1)

		if betweenTwoComponents:
			if len(selection)>1:
				componentsOnPath = Slots_max.getPathAlongLoop([selection[0], selection[-1]])
				pm.select(componentsOnPath)

		if autoAlign: #set coordinates for auto align:
			if len(selection)>1:

				point = pm.xform(selection, q=True, t=True, ws=True)
				#vertex point 1
				x1 = round(point[0], 4)
				y1 = round(point[1], 4)
				z1 = round(point[2], 4)

				#vertex point 2
				x2 = round(point[3], 4)
				y2 = round(point[4], 4)
				z2 = round(point[5], 4)

				#find the axis with the largest variance to determine direction.
				x = abs(x1-x2)
				y = abs(y1-y2)
				z = abs(z1-z2)

				maskEdge = pm.selectType (query=True, edge=True)
				if maskEdge:
					selection = pm.polyListComponentConversion(fromEdge=1, toVertexFace=1)

				vertex = selection[0] if selection else None
				if vertex is None:
					self.messageBox('Unable to get component path.')
					return

				vertexTangent = pm.polyNormalPerVertex(vertex, query=True, xyz=True)

				tx = abs(round(vertexTangent[0], 4))
				ty = abs(round(vertexTangent[1], 4))
				tz = abs(round(vertexTangent[2], 4))

				axis = max(x,y,z)
				tangent = max(tx,ty,tz)

				if autoAlign2Axes:
					if axis==x: #"yz"
						self.sb.toggleWidgets(tb.ctxMenu, setChecked='chk030-31', setUnChecked='chk029')
					if axis==y: #"xz"
						self.sb.toggleWidgets(tb.ctxMenu, setChecked='chk029,chk031', setUnChecked='chk030')
					if axis==z: #"xy"
						self.sb.toggleWidgets(tb.ctxMenu, setChecked='chk029-30', setUnChecked='chk031')
				else:
					if any ([axis==x and tangent==ty, axis==y and tangent==tx]): #"z"
						self.sb.toggleWidgets(tb.ctxMenu, setChecked='chk031', setUnChecked='chk029-30')
					if any ([axis==x and tangent==tz, axis==z and tangent==tx]): #"y"
						self.sb.toggleWidgets(tb.ctxMenu, setChecked='chk030', setUnChecked='chk029,chk031')
					if any ([axis==y and tangent==tz, axis==z and tangent==ty]): #"x"
						self.sb.toggleWidgets(tb.ctxMenu, setChecked='chk029', setUnChecked='chk030-31')
			else:
				self.messageBox('Operation requires a component selection.')
				return

		#align
		x = tb.ctxMenu.chk029.isChecked()
		y = tb.ctxMenu.chk030.isChecked()
		z = tb.ctxMenu.chk031.isChecked()
		avg = tb.ctxMenu.chk006.isChecked()
		loop = tb.ctxMenu.chk007.isChecked()

		if all ([x, not y, not z]): #align x
			self.alignVertices(mode=3,average=avg,edgeloop=loop)

		if all ([not x, y, not z]): #align y
			self.alignVertices(mode=4,average=avg,edgeloop=loop)

		if all ([not x, not y, z]): #align z
			self.alignVertices(mode=5,average=avg,edgeloop=loop)

		if all ([not x, y, z]): #align yz
			self.alignVertices(mode=0,average=avg,edgeloop=loop)

		if all ([x, not y, z]): #align xz
			self.alignVertices(mode=1,average=avg,edgeloop=loop)

		if all ([x, y, not z]): #align xy
			self.alignVertices(mode=2,average=avg,edgeloop=loop)

		if all ([x, y, z]): #align xyz
			self.alignVertices(mode=6,average=avg,edgeloop=loop)


	@Slots.hideMain
	def b000(self):
		'''Object Transform Attributes
		'''
		selection = list(rt.selection)
		if not selection:
			self.messageBox('<b>Nothing selected.</b><br>The operation requires a single selected object.')
			return

		obj = selection[0]

		props = ['pos.x', 'pos.y', 'pos.z', 'rotation.x_rotation', 'rotation.y_rotation', 'rotation.z_rotation', 
				'scale.x', 'scale.y', 'scale.z', 'center', 'pivot.x', 'pivot.y', 'pivot.z']
		attrs = {p:getattr(obj, p) for p in props}

		self.setAttributeWindow(obj, checkableLabel=True, **attrs)


	def b001(self):
		'''Match Scale
		'''
		selection = list(rt.selection)
		if not selection:
			self.messageBox('<b>Nothing selected.</b><br>The operation requires at least two selected object.')
			return

		frm = selection[0]
		to = selection[1:]

		Slots_maya.matchScale(to, frm)


	def lbl000(self):
		'''Transform Constraints: Disable All
		'''
		widgets = self.sb.transform.cmb001.ctxMenu.children_(of_type=['QCheckBox'])
		[w.setChecked(False) for w in widgets if w.isChecked()]


	def lbl001(self):
		'''Transform Tool Snapping: Disable All
		'''
		cmb = self.sb.transform.cmb003
		self.sb.toggleWidgets(setDisabled='chk021-23')
		cmb.setCurrentText('Off') if not any((state, cmb.menu_.chk021.isChecked(), cmb.menu_.chk023.isChecked())) else cmb.setCurrentText('On')


	def b002(self):
		'''Freeze Transformations
		'''
		maxEval('macros.run \"Animation Tools\" \"FreezeTransform\"')


	def b003(self):
		'''Center Pivot Object
		'''
		for obj in rt.selection:
			rt.toolMode.coordsys(obj)
			obj.pivot = obj.center

	
	def b005(self):
		'''Move To
		'''
		sel = [s for s in rt.getCurrentSelection()] #rebuild selection array in python.

		objects = sel[:-1]
		target = sel[-1]
		#move object(s) to center of the last selected items bounding box
		for obj in objects: 
			obj.center = target.center


	def b014(self):
		'''Center Pivot Component
		'''
		[pm.xform (s, centerPivot=1) for s in pm.ls (sl=1, objectsOnly=1, flatten=1)]
		# mel.eval("moveObjectPivotToComponentCentre;")


	def b015(self):
		'''Center Pivot World
		'''
		mel.eval("xform -worldSpace -pivots 0 0 0;")


	def b016(self):
		'''Set To Bounding Box
		'''
		mel.eval("bt_alignPivotToBoundingBoxWin;")


	def b017(self):
		'''Bake Pivot
		'''
		mel.eval("BakeCustomPivot;")


	def b032(self):
		'''Reset Pivot Transforms
		'''
		maxEval('''
			{ string $objs[] = `ls -sl -type transform -type geometryShape`;
			if (size($objs) > 0) { xform -cp; } manipPivot -rp -ro; };
			''')


	def setTransformSnap(self, ctx, state):
		'''Set the transform tool's move, rotate, and scale snap states.

		:Parameters:
			ctx (str): valid: 'move', 'scale', 'rotate'
			state (int): valid: 0=off, 1=relative, 2=absolute
		'''
		if ctx=='move':
			pm.manipMoveContext('Move', edit=1, snap=False if state==0 else True, snapRelative=True if state==1 else False) #state: 0=off, 1=relative, 2=absolute
			pm.texMoveContext('texMoveContext', edit=1, snap=False if state==0 else True) #uv move context

		elif ctx=='scale':
			pm.manipScaleContext('Scale', edit=1, snap=False if state==0 else True, snapRelative=True if state==1 else False) #state: 0=off, 1=relative, 2=absolute
			pm.texScaleContext('texScaleContext', edit=1, snap=False if state==0 else True) #uv scale context

		elif ctx=='rotate':
			pm.manipRotateContext('Rotate', edit=1, snap=False if state==0 else True, snapRelative=True if state==1 else False) #state: 0=off, 1=relative, 2=absolute
			pm.texRotateContext('texRotateContext', edit=1, snap=False if state==0 else True) #uv rotate context


	def setSnapState(self, fn, state):
		'''Grid and Snap Settings: Modify grid and snap states.

		:Parameters:
			fn (str): Snap string name.
			state (bool): Desired snap state.

		Valid fn arguments for snap name:
			Body Shapes: (1) 'Vertex_', 'Edge', 'Face_', 'End Edge', 'Edge Midpoint'
			NURBS: (2) 'CV', 'Curve Center', 'Curve Tangent', 'Curve End', 'Surface Normal', 'Point', 'Curve Normal', 'Curve Edge', 'Surface Center','Surface Edge'
			Point Cloud Shapes: (3) 'Point Cloud Vertex'
			Standard: (4,5,6,7) 'Grid Points', 'Pivot', 'Perpendicular', 'Vertex', 'Edge/Segment', 'Face', 'Grid Lines', 'Bounding Box', 'Tangent', 'Endpoint', 'Midpoint', 'Center Face'

		ex. setSnapState('Edge', True)
		'''
		snaps = {
			1:['Vertex_', 'Edge', 'Face_', 'End Edge', 'Edge Midpoint'], #Body Shapes
			2:['CV', 'Curve Center', 'Curve Tangent', 'Curve End', 'Surface Normal', 'Point', 'Curve Normal', 'Curve Edge', 'Surface Center','Surface Edge'], #NURBS
			3:['Point Cloud Vertex'], #Point Cloud Shapes
			4:['Grid Points', 'Pivot'], #Standard
			5:['Perpendicular', 'Vertex'], #Standard
			6:['Edge/Segment', 'Face'], #Standard
			7:['Grid Lines', 'Bounding Box', 'Tangent', 'Endpoint', 'Midpoint', 'Center Face'] #Standard
		}

		for category, lst in snaps.items():
			if fn in lst:
				index = lst.index(fn)+1 #add 1 to align with max array.
				rt.snapmode.setOSnapItemActive(category, index, state) #ie. rt.snapmode.setOSnapItemActive(3, 1, False) #'Point Cloud Shapes'->'Point Cloud Vertex'->Off
				print (fn, '|', state)


	def alignVertices(self, selection, mode):
		'''Align Vertices

		Align all vertices at once by putting each vert index and coordinates in a dict (or two arrays) then if when iterating through a vert falls within the tolerance specified in a textfield align that vert in coordinate. then repeat the process for the other coordinates x,y,z specified by checkboxes. using edges may be a better approach. or both with a subObjectLevel check
		create edge alignment tool and then use subObjectLevel check to call either that function or this one from the same buttons.
		to save ui space; have a single align button, x, y, z, and align 'all' checkboxes and a tolerance textfield.

		:Parameters:
			selection (list): vertex selection
			mode (int): valid values are: 0 (YZ), 1 (XZ), 2 (XY), 3 (X), 4 (Y), 5 (Z)

		notes:
		'vertex.pos.x = vertPosX' ect doesnt work. had to use maxscript
		'''
		# maxEval('undo "alignVertices" on')
		componentArray = selection.selectedVerts
		
		if len(componentArray) == 0:
			self.messageBox('No vertices selected.')
			return

		if len(componentArray) < 2:
			self.messageBox('Selection must contain at least two vertices.')
			return

		lastSelected = componentArray[-1]#3ds max re-orders array by vert index, so this doesnt work for aligning to last selected
		#~ print(lastSelected.pos)
		aX = lastSelected.pos[0]
		aY = lastSelected.pos[1]
		aZ = lastSelected.pos[2]
		
		for vertex in componentArray:
			#~ print(vertex.pos)
			vX = vertex.pos[0]
			vY = vertex.pos[1]
			vZ = vertex.pos[2]

			maxEval('global alignXYZ')
			
			if mode == 0: #align YZ
				maxEval('''
				fn alignXYZ mode vertex vX vY vZ aX aY aZ=
				(
					vertex.pos.x = vX
					vertex.pos.y = aY
					vertex.pos.z = aZ
				)
				''')
				
			if mode == 1: #align XZ
				maxEval('''
				fn alignXYZ mode vertex vX vY vZ aX aY aZ=
				(
					vertex.pos.x = aX
					vertex.pos.y = vY
					vertex.pos.z = aZ
				)
				''')
			
			if mode == 2: #align XY
				maxEval('''
				fn alignXYZ mode vertex vX vY vZ aX aY aZ=
				(
					vertex.pos.x = aX
					vertex.pos.y = aY
					vertex.pos.z = vZ
				)
				''')
			
			if mode == 3: #X
				maxEval('''
				fn alignXYZ mode vertex vX vY vZ aX aY aZ=
				(
					vertex.pos.x = aX
					vertex.pos.y = vY
					vertex.pos.z = vZ
				)
				''')
			
			if mode == 4: #Y
				maxEval('''
				fn alignXYZ mode vertex vX vY vZ aX aY aZ=
				(
					vertex.pos.x = vX
					vertex.pos.y = aY
					vertex.pos.z = vZ
				)
				''')
			
			if mode == 5: #Z
				maxEval('''
				fn alignXYZ mode vertex vX vY vZ aX aY aZ=
				(
					vertex.pos.x = vX
					vertex.pos.y = vY
					vertex.pos.z = aZ
				)
				''')
			
			print(100*"-")
			print("vertex.index:", vertex.index)
			print("position:", vX, vY, vZ)
			print("align:   ", aX, aY, aZ)
			
			rt.alignXYZ(mode, vertex, vX, vY, vZ, aX, aY, aZ)

			self.messageBox('{0}{1}{2}{3}'.format("result: ", vertex.pos[0], vertex.pos[1], vertex.pos[2]))


	def scaleObject(self, size, x, y ,z):
		'''
		:Parameters:
			size (float) = Scale amount
			x (bool): Scale in the x direction.
			y (bool): Scale in the y direction.
			z (bool): Scale in the z direction.

		Basically working except for final 'obj.scale([s, s, s])' command in python. variable definitions included for debugging.
		to get working an option is to use the maxEval method in the alignVertices function.
		'''
		textField_000 = 1.50
		isChecked_002 = True
		isChecked_003 = True
		isChecked_004 = True

		s = textField_000
		x = isChecked_002
		y = isChecked_003
		z = isChecked_004
		#-------------------------
		s = size
		selection = rt.selection

		for obj in selection:
			if (isChecked_002 and isChecked_003 and isChecked_004):
				obj.scale([s, s, s])
			if (not isChecked_002 and isChecked_003 and isChecked_004):
				obj.scale([1, s, s])
			if (isChecked_002 (not isChecked_003) and isChecked_004):
				obj.scale([s, 1, s])
			if (isChecked_002 and isChecked_003 (not isChecked_004)):
				obj.scale([s, s, 1])
			if (not isChecked_002 (not isChecked_003) and isChecked_004):
				obj.scale([1, 1, s])
			if (isChecked_002 (not isChecked_003) (not isChecked_004)):
				obj.scale([s, 1, 1])
			if (isChecked_002 and isChecked_003 and isChecked_004):
				obj.scale([1, s, 1])
			if (not isChecked_002 (not isChecked_003) (not isChecked_004)):
				obj.scale([1, 1, 1])


	def compareSize(self, obj1, obj2, factor):
		'''Compares two point3 sizes from obj bounding boxes.

		:Parameters:
			obj1 (obj): 
			obj2 (obj): 
			factor () = 
		'''
		maxEval('''
		s1 = obj1.max - obj1.min --determine bounding boxes
		s2 = obj2.max - obj2.min
		
		if (s2.x >= (s1.x*(1-factor)) AND s2.x <= (s1.x*(1+factor))) OR (s2.x >= (s1.y*(1-factor)) AND s2.x <= (s1.y*(1+factor))) OR (s2.x >= (s1.z*(1-factor)) AND s2.x <= (s1.z*(1+factor)))THEN
			if (s2.y >= (s1.y*(1-factor)) AND s2.y <= (s1.y*(1+factor))) OR (s2.y >= (s1.x*(1-factor)) AND s2.y <= (s1.x*(1+factor))) OR (s2.y >= (s1.z*(1-factor)) AND s2.y <= (s1.z*(1+factor))) THEN
				if (s2.z >= (s1.z*(1-factor)) AND s2.z <= (s1.z*(1+factor))) OR (s2.z >= (s1.x*(1-factor)) AND s2.z <= (s1.x*(1+factor))) OR (s2.z >= (s1.y*(1-factor)) AND s2.z <= (s1.y*(1+factor))) THEN
				(
					dbgSelSim ("  Size match on '" + obj1.name + "' with '" + obj2.name + "'")
					return true
				)
				else return false
			else return false
		else return false			
		''')









#module name
print (__name__)
# --------------------------------------------------------------------------------------------
# Notes
# --------------------------------------------------------------------------------------------

# deprecated:

# maxEval('max tti')

# maxEval('macros.run \"PolyTools\" \"TransformTools\")



	# def b000(self):
	# 	'''
	# 	Transform: negative
	# 	'''
	# 	#change the textfield to neg value and call performTransformations
	# 	textfield = float(self.sb.transform.s000.value())
	# 	if textfield >=0:
	# 		newText = -textfield
	# 		self.sb.transform.s000.setValue(newText)
	# 	self.performTransformations()


	# def b001(self):
	# 	'''
	# 	Transform: positive
	# 	'''
	# 	#change the textfield to pos value and call performTransformations
	# 	textfield = float(self.sb.transform.s000.value())
	# 	if textfield <0:
	# 		newText = abs(textfield)
	# 		self.sb.transform.s000.setValue(newText)
	# 	self.performTransformations()



# !/usr/bin/python
# coding=utf-8
import sys, os.path

from PySide2 import QtWidgets, QtCore

from pythontk import Iter, setAttributes


class Slots(QtCore.QObject):
	'''Provides methods that can be triggered by widgets in the ui.
	Parent to the 'Init' slot class, which is in turn, inherited by every other slot class.

	If you need to create a invokable method that returns some value, declare it as a slot, e.g.:
	@Slot(result=int, float)
	ex. def getFloatReturnInt(self, f):
			return int(f)
	'''
	def __init__(self, parent=None, *args, **kwargs):
		QtCore.QObject.__init__(self, parent)
		'''
		'''
		for a, v in kwargs.items():
			setattr(self, a, v)


	def hideMain(fn):
		'''A decorator that hides the stacked widget main window.
		'''
		def wrapper(self, *args, **kwargs):
			fn(self, *args, **kwargs) #execute the method normally.
			self.sb.parent().hide() #Get the state of the widget in the current ui and set any widgets (having the methods name) in child or parent ui's accordingly.
		return wrapper


	def objAttrWindow(self, obj, checkableLabel=False, fn=None, fn_args=[], **attributes):
		'''Launch a popup window containing the given objects attributes.

		:Parameters:
			obj (obj): The object to get the attributes of.
			checkableLabel (bool): Set the attribute labels as checkable.
			fn (method) = Set an alternative method to call on widget signal. ex. setParameterValuesMEL
					The first parameter of fn is always the given object. ex. fn(obj, {'attr':<value>})
			fn_args (list): Any additonal args to pass to fn.
			attributes (kwargs) = Explicitly pass in attribute:values pairs. Else, attributes will be pulled from mtk.Node.getAttributesMEL for the given obj.

		:Return:
			(obj) the menu widget. (use menu.childWidgets to get the menu's child widgets.)

		:Example: self.objAttrWindow(node, attrs, fn=mtk.setParameterValuesMEL, fn_args='transformLimits')
		:Example: self.objAttrWindow(transform[0], fn_args=['translateX','translateY','translateZ','rotateX','rotateY','rotateZ','scaleX','scaleY','scaleZ'], checkableLabel=True)
		'''
		import ast

		fn = fn if fn else setAttributes
		fn_args = Iter.makeList(fn_args) #assure that fn_args is a list.

		try: #get the objects name to as the window title:
			title = obj.name()
		except:
			try:
				title = obj.name
			except:
				title = str(obj)

		menu = self.sb.Menu(self.sb.parent(), menu_type='form', padding=2, title=title, position='cursorPos')

		for a, v in attributes.items():

			if isinstance(v, (float, int, bool)):
				if type(v)==int or type(v)==bool:
					s = menu.add('QSpinBox', label=a, checkableLabel=checkableLabel, setSpinBoxByValue_=v)

				elif type(v)==float:
					v = float(f'{v:g}') ##remove any trailing zeros from the float value.
					s = menu.add('QDoubleSpinBox', label=a, checkableLabel=checkableLabel, setSpinBoxByValue_=v, setDecimals=3)

				s.valueChanged.connect(lambda v, a=a: fn(obj, *fn_args, **{a:v}))

			else: #isinstance(v, (list, set, tuple)):
				w = menu.add('QLineEdit', label=a, checkableLabel=checkableLabel, setText=str(v))
				w.returnPressed.connect(lambda w=w, a=a: fn(obj, *fn_args, **{a:ast.literal_eval(w.text())}))

		self.sb.setStyle(menu.childWidgets)
		menu.show()

		return menu


	def messageBox(self, string, messageType='', location='topMiddle', timeout=1):
		'''Spawns a message box with the given text.
		Supports HTML formatting.
		Prints a formatted version of the given string to console, stripped of html tags, to the console.

		:Parameters:
			messageType (str): The message context type. ex. 'Error', 'Warning', 'Note', 'Result'
			location (str)(point) = move the messagebox to the specified location. Can be given as a qpoint or string value. default is: 'topMiddle'
			timeout (int): time in seconds before the messagebox auto closes.
		'''
		if messageType:
			string = '{}: {}'.format(messageType.capitalize(), string)

		if not hasattr(self, '_messageBox'):
			from uitk.widgets.messageBox import MessageBox
			self._messageBox = MessageBox(self.sb.parent().parent())

		self._messageBox.location = location
		self._messageBox.timeout = timeout

		from re import sub
		print(''+sub('<.*?>', '', string)+'') #strip everything between '<' and '>' (html tags)

		self._messageBox.setText(string)
		self._messageBox.exec_()


	# @classmethod
	# def progress(cls, fn):
	# 	'''A decorator for progressBar.
	#	Does not work with staticmethods.
	# 	'''
	# 	def wrapper(self, *args, **kwargs):
	# 		self.progressBar(fn(self, *args, **kwargs))
	# 	return wrapper

	def progressBar(self):
		'''
		'''
		try:
			return self._progressBar

		except AttributeError as error:
			from widgets.progressBar import ProgressBar
			self._progressBar = ProgressBar(self.sb.parent())

			try:
				self.sb.currentUi.progressBar.step1
			except AttributeError:
				pass

			return self._progressBar


	def invertOnModifier(self, value):
		'''Invert a numerical or boolean value if the alt key is pressed.

		:Parameters:
			value (int, float, bool) = The value to invert.
		
		:Return:
			(int, float, bool)
		'''
		modifiers = QtWidgets.QApplication.instance().keyboardModifiers()
		if not modifiers in (QtCore.Qt.AltModifier, QtCore.Qt.ControlModifier | QtCore.Qt.AltModifier):
			return value

		if type(value) in (int, float):
			result = abs(value) if value<0 else -value
		elif type(value)==bool:
			result = True if value else False

		return result

# --------------------------------------------------------------------------------------------









#module name
# print (__name__)
# --------------------------------------------------------------------------------------------
# Notes
# --------------------------------------------------------------------------------------------


# depricated: -----------------------------------



# !/usr/bin/python
# coding=utf-8
from uitk.slots.max import *
from uitk.slots.init import Init



class Init_max(Init, Slots_max):
	'''
	'''
	def __init__(self, *args, **kwargs):
		super().__init__(*args, **kwargs)

		try:
			self.sb.init.hud.shown.connect(self.construct_hud)

		except AttributeError: #(an inherited class)
			pass


	def construct_hud(self):
		'''Add current scene attributes to a lineEdit.
		Only those with relevant values will be displayed.
		'''
		hud = self.sb.init.hud

		try:
			selection = rt.selection
		except AttributeError:
			selection = None

		if selection:
			if len(selection) is 1:
				obj = selection[0]
				symmetry = obj.modifiers[rt.Symmetry]
				if symmetry:
					int_ = symmetry.axis
					axis = {0:'x', 1:'y', 2:'z'}
					hud.insertText('Symmetry Axis: <font style="color: Yellow;">{}'.format(axis[int_].upper())) #symmetry axis

				level = rt.subObjectLevel if rt.subObjectLevel else 0
				if level==0: #object level
					numberOfSelected = len(selection)
					if numberOfSelected<11:
						name_and_type = ['<font style="color: Yellow;">{0}<font style="color: LightGray;">:{1}'.format(obj.name, rt.classOf(obj.baseObject)) for obj in selection]
						name_and_type_str = str(name_and_type).translate(str.maketrans('', '', ',[]\'')) #format as single string. remove brackets, single quotes, and commas.
					else:
						name_and_type_str = '' #if more than 10 objects selected, don't list each object.
					hud.insertText('Selected: <font style="color: Yellow;">{0}<br/>{1}'.format(numberOfSelected, name_and_type_str)) #currently selected objects by name and type.

				elif level>0: #component level
					obj = selection[0]
					objType = rt.classOf(obj.baseObject)

					if objType==rt.Editable_Poly or objType==rt.Edit_Poly:
						if level==1: #get vertex info
							type_ = 'Verts'
							components = Slots_max.bitArrayToArray(rt.polyop.getVertSelection(obj))
							total_num = rt.polyop.getNumVerts(obj)

						elif level==2: #get edge info
							type_ = 'Edges'
							components = Slots_max.bitArrayToArray(rt.polyop.getEdgeSelection(obj))
							total_num = rt.polyop.getNumEdges(obj)

						elif level==3: #get border info
							type_ = 'Borders'
							# rt.polyop.SetSelection #Edge ((polyOp.getOpenEdges $) as bitarray)
							components = Slots_max.bitArrayToArray(rt.polyop.getBorderSelection(obj))
							total_num = rt.polyop.getNumBorders(obj)

						elif level==4: #get face info
							type_ = 'Faces'
							components = Slots_max.bitArrayToArray(rt.polyop.getFaceSelection(obj))
							total_num = rt.polyop.getNumFaces(obj)

						elif level==5: #get element info
							type_ = 'Elements'
							components = Slots_max.bitArrayToArray(rt.polyop.getElementSelection(obj))
							total_num = rt.polyop.getNumElements(obj)

						try:
							hud.insertText('Selected {}: <font style="color: Yellow;">{} <font style="color: LightGray;">/{}'.format(type_, len(components), total_num)) #selected components
						except NameError:
							pass

		method = self.sb.prevCommand
		if method:
			hud.insertText('Prev Command: <font style="color: Yellow;">{}'.format(method.__doc__))  #get button text from last used command

		# prevUi = self.sb.prevUiName(omitLevel=[0,1,2])
		# hud.insertText('Prev UI: {}'.format(prevUi.replace('_', '').title())) #get the last level 3 ui name string.

		# prevCamera = self.sb.prevCamera(docString=True)
		# hud.insertText('Prev Camera: {}'.format(prevCamera)) #get the previously used camera.









#module name
print (__name__)
# --------------------------------------------------------------------------------------------
# Notes
# --------------------------------------------------------------------------------------------




# --------------------------------------------------------------------------------------------
# deprecated:
# --------------------------------------------------------------------------------------------




# !/usr/bin/python
# coding=utf-8
from uitk.slots import Slots



class Main(Slots):
	'''
	'''
	def __init__(self, *args, **kwargs):
		super().__init__(*args, **kwargs)
		'''
		'''
		lw = self.sb.main_lower_submenu.lw000
		# print (lw.getItems())
		# print (lw.getItemWidgets())


	def lw000(self, w=None):
		'''
		'''
		lw = self.sb.main_lower_submenu.lw000

		if not w: # code here will run before each show event. generally used to refresh list contents. ------------------
			#command history
			recentCommandInfo = [m.__name__ for m in self.sb.prevCommands] #Get a list of any recently called method names.
			# w1 = lw.getItemWidgetsByText('Recent Commands')[0]
			w1 = lw.getItemsByText('Recent Commands')[0]
			# print (0, w1)
			lw._addList(w1)
			# print (1, w1.list)
			w2 = w1.list.add('QPushButton', setObjectName='b004', setText='Button 4')
			# [w1.list.add('QLabel', setText=m.__doc__) for m in recentCommandInfo]
			return

		# print (w.text(), w, w.list)
		if w.text=='Recent Commands':
			recentCommands = self.sb.prevCommands #Get a list of any previously called slot methods.
			method = recentCommands[index]
			if callable(method):
				method()




# !/usr/bin/python
# coding=utf-8
from PySide2 import QtWidgets, QtCore

from attributes import Attributes
from menu import MenuInstance



class Label(QtWidgets.QLabel, MenuInstance, Attributes):
	'''
	'''
	clicked = QtCore.Signal()
	released = QtCore.Signal()

	def __init__(self, parent=None, **kwargs):
		QtWidgets.QLabel.__init__(self, parent)

		self.setStyleSheet(parent.styleSheet()) if parent else None

		self.setTextFormat(QtCore.Qt.RichText)
		self.setAttributes(**kwargs)


	def mousePressEvent(self, event):
		'''
		:Parameters:
			event (QEvent) = 
		'''
		if event.button()==QtCore.Qt.LeftButton:
			self.clicked.emit()
			self.menu_.show()

		if event.button()==QtCore.Qt.RightButton:
			self.ctxMenu.show()

		QtWidgets.QLabel.mousePressEvent(self, event)


	def mouseReleaseEvent(self, event):
		'''
		:Parameters:
			event (QEvent) = 
		'''
		if event.button()==QtCore.Qt.LeftButton:
			self.released.emit()

		QtWidgets.QLabel.mouseReleaseEvent(self, event)









if __name__ == "__main__":
	import sys
	app = QtWidgets.QApplication.instance() or QtWidgets.QApplication(sys.argv) #return the existing QApplication object, or create a new one if none exists.

	w = Label(setText='QLabel', setVisible=True)
	w.resize(w.sizeHint().width(), 19)
	menuItem = w.menu_.add(Label, setText='menu item')
	ctxMenuItem = w.ctxMenu.add(Label, setText='context menu item')
	print (menuItem, ctxMenuItem)
	# w.show()
	sys.exit(app.exec_())



# -----------------------------------------------------------------------------
# Notes
# -----------------------------------------------------------------------------

'''
Promoting a widget in designer to use a custom class:
>	In Qt Designer, select all the widgets you want to replace, 
		then right-click them and select 'Promote to...'. 

>	In the dialog:
		Base Class:		Class from which you inherit. ie. QWidget
		Promoted Class:	Name of the class. ie. "MyWidget"
		Header File:	Path of the file (changing the extension .py to .h)  ie. myfolder.mymodule.mywidget.h

>	Then click "Add", "Promote", 
		and you will see the class change from "QWidget" to "MyWidget" in the Object Inspector pane.
'''

# deprecated:




# !/usr/bin/python
# coding=utf-8
from uitk.slots.maya import *
from uitk.slots.preferences import Preferences



class Preferences_maya(Preferences, Slots_maya):
	def __init__(self, *args, **kwargs):
		super().__init__(*args, **kwargs)

		self.sb.preferences.b010.setText('Maya Preferences')

		cmb000 = self.sb.preferences.draggable_header.ctxMenu.cmb000
		items = ['']
		cmb000.addItems_(items, '')

		cmb001 = self.sb.preferences.cmb001
		items = ['millimeter','centimeter','meter','kilometer','inch','foot','yard','mile']
		cmb001.addItems_(items)
		index = cmb001.items.index(pm.currentUnit(query=1, fullName=1, linear=1)) #get/set current linear value.
		cmb001.setCurrentIndex(index)

		cmb002 = self.sb.preferences.cmb002
		items = {'15 fps (game)':'game','24 fps (film)':'film','25 fps (pal)':'pal','30 fps (ntsc)':'ntsc','48 fps (show)':'show','50 fps (palf)':'palf','60 fps (ntscf)':'ntscf'}
		cmb002.addItems_(items)
		index = cmb002.items.index(pm.currentUnit(query=1, fullName=1, time=1)) #get/set current time value.
		cmb002.setCurrentIndex(index)

		cmb003 = self.sb.preferences.cmb003
		from PySide2 import QtWidgets, QtCore
		items = QtWidgets.QStyleFactory.keys() #get styles from QStyleFactory
		cmb003.addItems_(items)
		index = cmb003.findText(QtWidgets.QApplication.instance().style().objectName(), QtCore.Qt.MatchFixedString) #get/set current value
		cmb003.setCurrentIndex(index)


	def cmb000(self, index=-1):
		'''Editors
		'''
		cmb = self.sb.preferences.draggable_header.ctxMenu.cmb000

		if index>0:
			text = cmb.items[index]
			if text=='':
				pass
			cmb.setCurrentIndex(0)


	def cmb001(self, index=-1):
		'''Set Working Units: Linear
		'''
		cmb = self.sb.preferences.cmb001

		if index>0:
			pm.currentUnit(linear=cmb.items[index]) #millimeter | centimeter | meter | kilometer | inch | foot | yard | mile


	def cmb002(self, index=-1):
		'''Set Working Units: Time
		'''
		cmb = self.sb.preferences.cmb002

		if index>0:
			pm.currentUnit(time=cmb.items[index].split()[-1]) #game | film | pal | ntsc | show | palf | ntscf


	def b001(self):
		'''Color Settings

		'''
		pm.mel.colorPrefWnd()


	def b008(self):
		'''Hotkeys

		'''
		pm.mel.HotkeyPreferencesWindow()


	def b009(self):
		'''Plug-In Manager

		'''
		pm.mel.PluginManager()


	def b010(self):
		'''Settings/Preferences

		'''
		pm.mel.PreferencesWindow()


	@staticmethod
	def loadPlugin(plugin):
		'''Loads A Plugin.
		
		:Parameters:
			plugin (str): The desired plugin to load.

		ex. loadPlugin('nearestPointOnMesh')
		'''
		not pm.pluginInfo(plugin, query=True, loaded=True) and pm.loadPlugin(plugin)









#module name
print (__name__)
# --------------------------------------------------------------------------------------------
# Notes
# --------------------------------------------------------------------------------------------



#deprecated

# def cmb000(self):
# 	'''
# 	Custom Menu Set
# 	'''
# 	cmb = self.sb.preferences.draggable_header.ctxMenu.cmb000
	
# 	items = ['Modeling', 'Normals', 'Materials', 'UV'] #combobox list menu corresponding to the button text sets.
# 	contents = cmb.addItems_(items, 'Menu Sets')

# 	if not index:
		# index = cmb.currentIndex()
# 	buttons = self.getObjects(sb.getUi('main'), 'v000-11') #the ui in which the changes are to be made.
# 	for i, button in enumerate(buttons):
# 		if index==1: #set the text for each button.
# 			button.setText(['','','','','','','','','','','',''][i])

# 		if index==2:
# 			button.setText(['','','','','','','','','','','',''][i])


# !/usr/bin/python
# coding=utf-8
from uitk.slots.blender import *
from uitk.slots.crease import Crease



class Crease_blender(Crease, Slots_blender):
	def __init__(self, *args, **kwargs):
		Slots_blender.__init__(self, *args, **kwargs)
		Crease.__init__(self, *args, **kwargs)

		self.creaseValue = 10

		cmb = self.sb.crease.draggable_header.ctxMenu.cmb000
		items = []
		cmb.addItems_(items, 'Crease Editors:')


	def cmb000(self, index=-1):
		'''Editors
		'''
		cmb = self.sb.crease.draggable_header.ctxMenu.cmb000

		if index>0:
			text = cmb.items[index]
			if text=='Crease Set Editor':
				from maya.app.general import creaseSetEditor
				creaseSetEditor.showCreaseSetEditor()

			cmb.setCurrentIndex(0)


	def tb000(self, state=None):
		'''Crease
		'''
		tb = self.sb.crease.tb000

		creaseAmount = float(tb.ctxMenu.s003.value())
		normalAngle = int(tb.ctxMenu.s004.value()) 

		if tb.ctxMenu.chk011.isChecked(): #crease: Auto
			angleLow = int(tb.ctxMenu.s005.value()) 
			angleHigh = int(tb.ctxMenu.s006.value()) 

			mel.eval("PolySelectConvert 2;") #convert selection to edges
			contraint = pm.polySelectConstraint( mode=3, type=0x8000, angle=True, anglebound=(angleLow, angleHigh) ) # to get edges with angle between two degrees. mode=3 (All and Next) type=0x8000 (edge). 

		operation = 0 #Crease selected components
		pm.polySoftEdge (angle=0, constructionHistory=0) #Harden edge normal
		if tb.ctxMenu.chk002.isChecked():
			objectMode = pm.selectMode (query=True, object=True)
			if objectMode: #if in object mode,
				operation = 2 #2-Remove all crease values from mesh
			else:
				operation = 1 #1-Remove crease from sel components
				pm.polySoftEdge (angle=180, constructionHistory=0) #soften edge normal

		if tb.ctxMenu.chk004.isChecked(): #crease vertex point
			pm.polyCrease (value=creaseAmount, vertexValue=creaseAmount, createHistory=True, operation=operation)
		else:
			pm.polyCrease (value=creaseAmount, createHistory=True, operation=operation) #PolyCreaseTool;

		if tb.ctxMenu.chk005.isChecked(): #adjust normal angle
			pm.polySoftEdge (angle=normalAngle)

		if tb.ctxMenu.chk011.isChecked(): #crease: Auto
			pm.polySelectConstraint( angle=False ) # turn off angle constraint


	@Slots_blender.undoChunk
	def b002(self):
		'''Transfer Crease Edges
		'''
		# an updated version of this is in the maya python projects folder. transferCreaseSets.py
		# the use of separate buttons for donor and target mesh are deprecated.
		# add pm.polySoftEdge (angle=0, constructionHistory=0); #harden edge, when applying crease.
		
		creaseSet = str(self.sb.crease.b000.text())
		newObject = str(self.sb.crease.b001.text())

		sets = pm.sets (creaseSet, query=1)

		setArray = []
		for set_ in sets:
			name = str(set_)
			setArray.append(name)
		print(setArray)

		# pm.undoInfo (openChunk=1)
		for set_ in setArray:
			oldObject = ''.join(set_.partition('.')[:1]) #ex. pSphereShape1 from pSphereShape1.e[260:299]
			pm.select (set_, replace=1)
			value = pm.polyCrease (query=1, value=1)[0]
			name = set_.replace(oldObject, newObject)
			pm.select (name, replace=1)
			pm.polyCrease (value=value, vertexValue=value, createHistory=True)
			# print("crease:", name)
		# pm.undoInfo (closeChunk=1)

		self.sb.toggleWidgets(setDisabled='b052', setUnChecked='b000')#,self.sb.crease.b001])
		self.sb.crease.b000.setText("Crease Set")
		# self.sb.crease.b001.setText("Object")


	def getCreasedEdges(self, edges):
		'''Return any creased edges from a list of edges.

		:Parameters:
			edges (str)(obj)(list): The edges to check crease state on.

		:Return:
			(list) edges.
		'''
		creased_edges = [e for e in pm.ls(edges, flatten=1) if pm.polyCrease(e, q=1, value=1)[0] > 0]

		return creased_edges









#module name
print (__name__)
# --------------------------------------------------------------------------------------------
# Notes
# --------------------------------------------------------------------------------------------
#b008, b010, b011, b019, b024-27, b058, b059, b060


# !/usr/bin/python
# coding=utf-8
from uitk.slots import Slots


class Edit(Slots):
	'''
	'''
	def __init__(self, *args, **kwargs):
		super().__init__(*args, **kwargs)
		'''
		'''
		dh = self.sb.edit.draggable_header
		dh.ctxMenu.add(self.sb.ComboBox, setObjectName='cmb000', setToolTip='Editors')

		cmb001 = self.sb.edit.cmb001
		cmb001.beforePopupShown.connect(self.cmb001) #refresh comboBox contents before showing it's popup.

		tb001 = self.sb.edit.tb001
		tb001.ctxMenu.add('QCheckBox', setText='For All Objects', setObjectName='chk018', setChecked=True, setToolTip='Delete history on All objects or just those selected.')
		tb001.ctxMenu.add('QCheckBox', setText='Delete Unused Nodes', setObjectName='chk019', setChecked=True, setToolTip='Delete unused nodes.')
		tb001.ctxMenu.add('QCheckBox', setText='Delete Deformers', setObjectName='chk020', setToolTip='Delete deformers.')
		tb001.ctxMenu.add('QCheckBox', setText='Optimize Scene', setObjectName='chk030', setToolTip='Remove unused scene objects.')

		tb002 = self.sb.edit.tb002
		tb002.ctxMenu.add('QCheckBox', setText='Delete Edge Loop', setObjectName='chk001', setToolTip='Delete the edge loops of any edges selected.')
		tb002.ctxMenu.add('QCheckBox', setText='Delete Edge Ring', setObjectName='chk000', setToolTip='Delete the edge rings of any edges selected.')

		tb003 = self.sb.edit.tb003
		tb003.ctxMenu.add('QCheckBox', setText='-', setObjectName='chk006', setChecked=True, setToolTip='Perform delete along negative axis.')
		tb003.ctxMenu.add('QRadioButton', setText='X', setObjectName='chk007', setChecked=True, setToolTip='Perform delete along X axis.')
		tb003.ctxMenu.add('QRadioButton', setText='Y', setObjectName='chk008', setToolTip='Perform delete along Y axis.')
		tb003.ctxMenu.add('QRadioButton', setText='Z', setObjectName='chk009', setToolTip='Perform delete along Z axis.')
		self.sb.connect('chk006-9', 'toggled', self.chk006_9, tb003.ctxMenu)

		tb004 = self.sb.edit.tb004
		tb004.ctxMenu.add('QCheckBox', setText='All Nodes', setObjectName='chk026', setToolTip='Effect all nodes or only those currently selected.')
		tb004.ctxMenu.add('QCheckBox', setText='UnLock', setObjectName='chk027', setChecked=True, setToolTip='Unlock nodes (else lock).')
		tb004.ctxMenu.chk027.toggled.connect(lambda state: tb004.setText('Unlock Nodes' if state else 'Lock Nodes'))


	def draggable_header(self, state=None):
		'''Context menu
		'''
		dh = self.sb.edit.draggable_header


	def chk006_9(self):
		'''Set the toolbutton's text according to the checkstates.
		'''
		tb = self.sb.edit.tb003
		axis = self.sb.getAxisFromCheckBoxes('chk006-9', tb.ctxMenu)
		tb.setText('Delete '+axis)

		


# !/usr/bin/python
# coding=utf-8
from uitk.slots.max import *
from uitk.slots.edit import Edit



class Edit_max(Edit, Slots_max):
	def __init__(self, *args, **kwargs):
		super().__init__(*args, **kwargs)

		cmb = self.sb.edit.draggable_header.ctxMenu.cmb000
		items = []
		cmb.addItems_(items, 'Max Editors')

		ctx = self.sb.edit.tb000.ctxMenu
		if not ctx.containsMenuItems:
			ctx.add('QCheckBox', setText='N-Gons', setObjectName='chk002', setToolTip='Find N-gons.')
			ctx.add('QCheckBox', setText='Isolated Vertex', setObjectName='chk003', setChecked=True, setToolTip='Find isolated vertices within specified angle threshold.')
			ctx.add('QSpinBox', setPrefix='Loose Vertex Angle: ', setObjectName='s006', setMinMax_='1-360 step1', setValue=15, setToolTip='Loose vertex search: Angle Threshold.')
			ctx.add('QCheckBox', setText='Repair', setObjectName='chk004', setToolTip='Repair matching geometry. (else: select)')

		ctx = self.sb.edit.tb002.ctxMenu
		ctx.chk000.setDisabled(True) #disable: Delete Edge Ring.


	def cmb000(self, index=-1):
		'''Editors
		'''
		cmb = self.sb.edit.draggable_header.ctxMenu.cmb000

		if index>0:
			text = cmb.items[index]
			if text=='':
				pass
			cmb.setCurrentIndex(0)


	@Slots_max.attr
	def cmb001(self, index=-1):
		'''Object History Attributes
		'''
		cmb = self.sb.edit.cmb001

		sel = list(rt.selection)
		if sel:
			items = ['{}.{}'.format(sel[0].name, m.name) for m in sel[0].modifiers]
			items.append('{}.{}'.format(sel[0].name, 'baseObject'))
		else:
			items = ['No selection.']
		cmb.addItems_(items, 'History')

		cmb.setCurrentIndex(0)
		if index>0:
			if cmb.items[index]!='No selection.':
				objName, attrName = cmb.items[index].split('.')
				obj = rt.getNodeByName(objName)
				return getattr(obj, attrName)


	def tb000(self, state=None):
		'''Mesh Cleanup
		'''
		tb = self.sb.edit.tb000

		isolatedVerts = tb.ctxMenu.chk003.isChecked() #isolated vertices
		edgeAngle = tb.ctxMenu.s006.value()
		nGons = tb.ctxMenu.chk002.isChecked() #n-sided polygons
		repair = tb.ctxMenu.chk004.isChecked() #attempt auto repair errors

		self.meshCleanup(isolatedVerts=isolatedVerts, edgeAngle=edgeAngle, nGons=nGons, repair=repair)


	def tb001(self, state=None):
		'''Delete History
		'''
		tb = self.sb.edit.tb001

		all_ = tb.ctxMenu.chk018.isChecked()
		unusedNodes = tb.ctxMenu.chk019.isChecked()
		deformers = tb.ctxMenu.chk020.isChecked()

		objects = pm.ls(selection=1)
		if all_:
			objects = pm.ls(typ="mesh")

		for obj in objects:
			try:
				if all_:
					pm.delete (obj, constructionHistory=1)
				else:
					pm.bakePartialHistory (obj, prePostDeformers=1)
			except:
				pass
		if unusedNodes:
			maxEval('hyperShadePanelMenuCommand("hyperShadePanel1", "deleteUnusedNodes");')

		#display viewPort messages
		if all_:
			if deformers:
				self.messageBox('Delete <hl>All</hl> History.')
				return
			else:
				self.messageBox('Delete <hl>All Non-Deformer</hl> History.')
				return
		else:
			if deformers:
				self.messageBox('Delete history on '+str(objects))
				return
			else:
				self.messageBox('Delete <hl>Non-Deformer</hl> history on '+str(objects))
				return


	def tb002(self, state=None):
		'''Delete
		'''
		tb = self.sb.edit.tb002

		level = rt.subObjectLevel

		for obj in rt.selection:
			if level==1: #vertices
				obj.EditablePoly.remove(selLevel='Vertex', flag=1)

			if level==2: #edges
				edges = rt.polyop.getEdgeSelection(obj)
				verts = rt.polyop.getVertsUsingEdge(obj, edges)
				rt.polyop.setVertSelection(obj, verts) #set vertex selection to be used by meshCleanup
				
				obj.EditablePoly.remove(selLevel='Edge', flag=1)
				self.meshCleanup(isolatedVerts=1, repair=1) #if any isolated verts exist: delete them

			if level==4: #faces
				faces = rt.polyop.getFaceSelection(obj)
				rt.polyop.deleteFaces(obj, faces, delIsoVerts=1)


	@Slots.hideMain
	def b001(self):
		'''Object History Attributes: get most recent node
		'''
		selection = rt.modPanel.getCurrentObject()
		if not selection:
			self.messageBox('Operation requires a single selected object.')
			return

		self.setAttributeWindow(selection, checkableLabel=True)


	def b021(self):
		'''Tranfer Maps
		'''
		maxEval('performSurfaceSampling 1;')


	def b022(self):
		'''Transfer Vertex Order
		'''
		maxEval('TransferVertexOrder;')


	def b023(self):
		'''Transfer Attribute Values
		'''
		maxEval('TransferAttributeValues;')


	def b027(self):
		'''Shading Sets
		'''
		print('no function')


	def findNGons(self, obj):
		'''Get a list of faces of a given object having more than four sides.

		:Parameters:
			obj (obj): polygonal object.

		:Return:
			(list) list containing any found N-Gons		
		'''
		faces = Slots_max.getComponents(obj, 'faces')

		Slots_max.setSubObjectLevel(4)
				
		nGons = [f for f in faces if rt.polyop.getFaceDeg(obj, f)>4]
		return nGons


	def getVertexVectors(self, obj, vertices):
		'''Generator to query vertex vector angles.

		ex.
		vectors = getVertexVectors(obj, vertices)
		for _ in range(len(vertices)):
			vector = vectors.next()
		'''
		for vertex in vertices:
			try:
				edges = Slots_max.bitArrayToArray(rt.polyop.getEdgesUsingVert(obj, vertex)) #get the edges that use the vertice
			except:
				edges = Slots_max.bitArrayToArray(rt.getEdgesUsingVert(obj, vertex)) #get the edges that use the vertice

			if len(edges)==2:
				try:
					vertexPosition = rt.polyop.getVert(obj, vertex)
				except:
					vertexPosition = rt.getVert(obj, vertex)

				try:
					edgeVerts = Slots_max.bitArrayToArray([rt.polyop.getVertsUsingEdge(obj, e) for e in edges])
				except:
					edgeVerts = Slots_max.bitArrayToArray([rt.getVertsUsingEdge(obj, e) for e in edges])

				edgeVerts = [v for v in edgeVerts if not v==vertex]

				try:
					vector1 = rt.normalize(rt.polyop.getVert(obj, edgeVerts[0]) - vertexPosition)
					vector2 = rt.normalize(rt.polyop.getVert(obj, edgeVerts[1]) - vertexPosition)
				except:
					vector1 = rt.normalize(rt.getVert(obj, edgeVerts[0]) - vertexPosition)
					vector2 = rt.normalize(rt.getVert(obj, edgeVerts[1]) - vertexPosition)

				vector = rt.length(vector1 + vector2)
				yield vector


	def findIsolatedVertices(self, obj):
		'''Get a list of isolated vertices of a given object.

		:Parameters:
			obj (obj): polygonal object.

		:Return:
			(list) list containing any found isolated verts.		
		'''
		vertices = Slots_max.getComponents(obj, 'vertices') #get all vertices for the given object

		isolatedVerts=[]
		vectors = self.getVertexVectors(obj, vertices)
		for _ in range(len(vertices)):
			vector = vectors.next()
			if vector and vector <= float(edgeAngle) / 50:
				isolatedVerts.append(vector)

		return isolatedVerts


	def meshCleanup(self, isolatedVerts=False, edgeAngle=10, nGons=False, repair=False):
		'''Find mesh artifacts.

		:Parameters:
			isolatedVerts (bool): find vertices with two edges which fall below a specified angle.
			edgeAngle (int): used with isolatedVerts argument to specify the angle tolerance
			nGons (bool): search for n sided polygon faces.
			repair (bool): delete or auto repair any of the specified artifacts 
		'''
		for obj in rt.selection:
			if not rt.classof(obj)==rt.Editable_poly:
				print('Error: '+obj.name+' isn\'t an editable poly or nothing is selected.')

			obj.selectMode = 2 #multi-component selection preview

			if nGons: #Convert N-Sided Faces To Quads
				_nGons = self.findNGons(obj)

				print('Found '+str(len(_nGons))+' N-gons.')

				if repair:
					maxEval('macros.run \"Modifiers\" \"QuadifyMeshMod\"') #add the quadify mesh modifier to the stack
				else: #Find and select N-gons	
					rt.setFaceSelection(obj, _nGons)


			if isolatedVerts: #delete loose vertices
				_isolatedVerts = self.findIsolatedVertices(obj)

				Slots_max.undo(True)
				try:
					rt.polyop.setVertSelection(obj, _isolatedVerts)
				except:
					rt.setVertSelection(obj, _isolatedVerts)

				print('Found '+str(len(_isolatedVerts))+' isolated vertices.')
				
				if repair:
					obj.EditablePoly.remove(selLevel='Vertex', flag=1)
					obj.selectMode = 0 #multi-component selection preview off
				rt.redrawViews()
				Slots_max.undo(False)


	def tb003(self, state=None):
		'''Delete Along Axis
		'''
		tb = self.sb.edit.tb003

		# selection = pm.ls(sl=1, objectsOnly=1)
		# axis = self.sb.getAxisFromCheckBoxes('chk006-9', tb.ctxMenu)

		# pm.undoInfo(openChunk=1)
		# for obj in selection:
		# 	self.deleteAlongAxis(obj, axis) #Slots_max.deleteAlongAxis - no max version.
		# pm.undoInfo(closeChunk=1)


	def b000(self):
		'''Clean: Repair
		'''
		self.cleanGeometry(repair=True, nsided=True, concave=False, nonplanar=False, zeroGeom=False, zeroEdge=False, zeroMap=False, 
							sharedUVs=False, nonmanifold=True, invalidComponents=False, splitNonManifoldVertex=False)


	@Slots.hideMain
	def b001(self):
		'''Object History Attributes: get most recent node
		'''
		cmb = self.sb.edit.cmb001
		self.cmb001() #refresh the contents of the combobox.

		items = pm.ls(cmb.items[-1])
		if items:
			self.setAttributeWindow(items, checkableLabel=True)
		else:
			self.messageBox('Found no items to list the history for.')
			return


	def b021(self):
		'''Tranfer Maps
		'''
		pm.mel.performSurfaceSampling(1)


	def b022(self):
		'''Transfer Vertex Order
		'''
		pm.mel.TransferVertexOrder()


	def b023(self):
		'''Transfer Attribute Values
		'''
		pm.mel.TransferAttributeValues()


	def b027(self):
		'''Shading Sets
		'''
		pm.mel.performTransferShadingSets(0)


	def compareMesh(self, obj1, obj2, factor):
		'''Compares vert/face/edges counts.

		:Parameters:
			obj1 (obj): 
			obj2 (obj): 
			factor () = 
		'''
		maxEval('''
		if superclassof obj1 == GeometryClass then	--if object is a geometry type (mesh)
		(
			o1v = obj1.mesh.verts.count	--store object 1 and 2 vert / face / edge counts
			o1f = obj1.mesh.faces.count
			o1e = obj1.mesh.edges.count
			o2v = obj2.mesh.verts.count
			o2f = obj2.mesh.faces.count
			o2e = obj2.mesh.edges.count
			
			if o2v >= o1v*(1-factor) AND o2v <= o1v*(1+factor) THEN 		--simpler than it looks. the 'factor' aka 'ratio' in the ui is 
				if o2f >= o1f*(1-factor) AND o2f <= o1f*(1+factor) THEN		--allows for slop in the comparison. 0.1 is a 10% difference so comparing
					if o2e >= o1e*(1-factor) AND o2e <= o1e*(1+factor) THEN	--face*(1-factor) is the same as saying face*0.9 in our example
					(
						dbgSelSim ("  Mesh match on '" + obj1.name + "' with '" + obj2.name + "' | V: " + o1v as string + ", " + o2v as string + " | F: " + o1f as string + ", " + o2f as string + " | E: " + o1e as string + ", " + o2e as string)
						return true
					)
					else return false
				else return false
			else return false
		)
		else if superclassof obj1 == shape then
		(
			o1v = numKnots obj1
			o2v = numKnots obj2
			
			if o2v >= o1v*(1-factor) AND o2v <= o1v*(1+factor) THEN
				return true
			else return false
		)
		''')









#module name
print (__name__)
# --------------------------------------------------------------------------------------------
# Notes
# --------------------------------------------------------------------------------------------

# maxEval('max array')

# maxEval('max mirror')


# !/usr/bin/python
# coding=utf-8
from PySide2 import QtCore


class StyleSheet(QtCore.QObject):
	'''# css commenting:
		/* multi-line */

	# setting a property:
		app.setStyleSheet("QLineEdit#name[prop=true] {background-color:transparent;}") #does't effect the lineEdit with objectName 'name' if that buttons property 'styleSheet' is false (c++ lowercase). 
		widget.setProperty('prop', True)

	# gradients:
		background-color: qlineargradient(x1: 0, y1: 0, x2: 0, y2: 1, stop: 0 #E0E0E0, stop: 1 #FFFFFF);

	# Set multiple widgets:
		QComboBox:!editable, QComboBox::drop-down:editable { ... }

	# Set the style for a specific widget:
		QWidget#mainWindow ('#' syntax, followed by the widget's objectName)
	'''
	_colorValues = {
			'standard': {
				'BACKGROUND'		: 'rgb(100,100,100)',
				'BACKGROUND_ALPHA'	: 'rgba(100,100,100,{ALPHA})',
				'PRESSED'			: 'rgb(125,125,125)',
				'HIGHLIGHT'			: 'yellow',
				'HOVER'				: 'rgb(82,133,166)',
				'TEXT'				: 'white',
				'TEXT_CHECKED'		: 'black',
				'TEXT_DISABLED'		: 'gray',
				'TEXT_HOVER'		: 'white',
				'TEXT_BACKGROUND'	: 'rgb(50,50,50)',
				'BORDER'			: 'rgb(50,50,50)',
			},

			'dark': {
				'BACKGROUND'		: 'rgb(60,60,60)',
				'BACKGROUND_ALPHA'	: 'rgba(60,60,60,{ALPHA})',
				'PRESSED'			: 'rgb(125,125,125)',
				'HIGHLIGHT'			: 'yellow',
				'HOVER'				: 'rgb(82,133,166)',
				'TEXT'				: 'rgb(185,185,185)',
				'TEXT_CHECKED'		: 'black',
				'TEXT_DISABLED'		: 'rgba(185,185,185,50)',
				'TEXT_HOVER'		: 'white',
				'TEXT_BACKGROUND'	: 'rgb(50,50,50)',
				'BORDER'			: 'rgb(40,40,40)',
			}
		}

	_styleSheets = {
		# 'QMainWindow': '''
		# 	QMainWindow {
		# 		background: {BACKGROUND};
		# 		color: {TEXT};
		# 		border: 1px solid {BORDER};
		# 	}
		# 	''',

		'QWidget': '''
			QWidget {
				background: {BACKGROUND_ALPHA};
			}

			QWidget#mainWindow {
				background: {BACKGROUND_ALPHA};
			} 

			QWidget::item:selected {
				background: {HOVER};
			}
			''',

		'QStackedWidget': '''
			QStackedWidget {

			}
			''',

		'QPushButton': '''
			QPushButton {
				border-style: outset;
				border-radius: 1px;
				border: 1px solid {BORDER};
				padding: 0px 5px 0px 5px; /* top, right, bottom, left */
				background-color: {BACKGROUND};
				color: {TEXT};
			}

			QPushButton::enabled {
				color: {TEXT};
			}

			QPushButton::disabled {
				color: {TEXT_DISABLED};
			}

			QPushButton::checked {
				background-color: {HOVER};
				color: {TEXT_CHECKED};
			}

			QPushButton::hover {
				background-color: {HOVER};
				color: {TEXT_HOVER};
			}

			QPushButton::checked::hover {
				background-color: {HOVER};
				color: {TEXT_CHECKED};
			}

			QPushButton::pressed {
				background-color: {PRESSED};
				color: {TEXT};
			}

			QPushButton:flat {
				border: none; /* no border for a flat push button */
			}

			QPushButton:default {
				border-color: navy; /* make the default button prominent */
			} 
			''',

		'QToolButton': '''
			QToolButton {
				border-style: outset;
				border-radius: 1px;
				border: 1px solid {BORDER};
				padding: 0px 5px 0px 5px; /* top, right, bottom, left */
				background-color: {BACKGROUND}; /* The background will not appear unless you set the border property. */
				color: {TEXT};
			}

			QToolButton::enabled {
				color: {TEXT};
			}

			QToolButton::disabled {
				color: {TEXT_DISABLED};
			}

			QToolButton::hover {   
				background-color: {HOVER};
				color: {TEXT_HOVER};
			}

			QToolButton::checked {
				background-color: {HOVER};
				color: {TEXT_CHECKED};
			}

			QToolButton::checked::hover {
				background-color: {HOVER};
				color: {TEXT_CHECKED};
			}

			QToolButton::pressed, QToolButton::menu-button:pressed {   
				background-color: {PRESSED};
				color: {TEXT};
			}

			QToolButton[popupMode="1"] { /* only for MenuButtonPopup */
				padding-right: 2px; /* make way for the popup button */
			}

			QToolButton:open { /* when the button has its menu open */
				background-color: dark gray;
			}

			/* popupMode set to DelayedPopup or InstantPopup */
			QToolButton::menu-indicator {
				image: none;
				subcontrol-origin: padding;
				subcontrol-position: bottom right;
				padding: 0px 5px 5px 0px; /* top, right, bottom, left */
			}

			QToolButton::menu-indicator:pressed, QToolButton::menu-indicator:open {
				position: relative;
				top: 2px; left: 2px; /* shift the arrow by 2 px */
			}

			/* When the Button displays arrows, the ::up-arrow, ::down-arrow, ::left-arrow and ::right-arrow subcontrols are used. */
			QToolButton::down-arrow, QToolButton::up-arrow, QToolButton::left-arrow, QToolButton::right-arrow {
				image: none;
				padding: 0px 15px 0px 0px; /* top, right, bottom, left */
			}

			QToolButton::down-arrow:hover, QToolButton::up-arrow:hover, QToolButton::left-arrow:hover, QToolButton::right-arrow:hover {
				background-color: {HOVER};
				padding: 0px 5px 0px 0px; /* top, right, bottom, left */
			}

			/* the subcontrols below are used only in the MenuButtonPopup mode */
			QToolButton::menu-button {
				border: 1px solid {TEXT};
				margin: 4px 2px 4px 0px; /* top, right, bottom, left */
			}

			QToolButton::menu-button::enabled {
				color: {TEXT};
			}

			QToolButton::menu-button::disabled {
				color: {TEXT_DISABLED};
				border: 1px solid transparent;
			}

			QToolButton::menu-button:hover{
				border: 1px solid {TEXT_HOVER};
			}

			QToolButton::menu-button:pressed {
				background-color: transparent;
				border: 1px solid transparent;
			}

			QToolButton::menu-arrow {
				image: none;
			}

			QToolButton::menu-arrow:open {

			} 
			''',

		'QAbstractButton': '''
			QAbstractButton:hover {
				background: {BACKGROUND};
			}

			QAbstractButton:pressed {
				background: {PRESSED};
			} 
			''',

		'QComboBox': '''
			QComboBox {
				background: {BACKGROUND};
				color: {TEXT};
				border: 1px solid {BORDER};
				padding: 1px 18px 1px 3px; /* top, right, bottom, left */
				/* border-radius: 1px; */
				/* min-width: 0em; */
			}

			QComboBox QAbstractItemView {
			min-width: 150px;
			}

			QComboBox::hover {
				background: {HOVER};
				color: {TEXT_HOVER};
				border: 1px solid {BORDER};
			}

			QComboBox::open {
				background: {BACKGROUND};
				color: {TEXT};
				border: 1px solid {BORDER};
				selection-background-color: {HOVER};
				selection-color: {TEXT_CHECKED};
			}

			QComboBox:on { /* shift the text when the popup opens */
				padding-top: 3px;
				padding-left: 4px;
			}

			QComboBox::down-arrow {
				width: 0px;
				height: 0px;
				border: transparent;
				background: {BACKGROUND};
			}

			QComboBox::drop-down {
				border: transparent;
				background: {BACKGROUND};
				subcontrol-origin: padding;
				subcontrol-position: top right;
				width: 0px;

				border-left-width: 1px;
				border-left-color: {TEXT_CHECKED};
				border-left-style: solid; /* just a single line */
				border-top-right-radius: 1px; /* same radius as the QComboBox */
				border-bottom-right-radius: 1px;
			} 
			''',

		'QSpinBox': '''
			QSpinBox {
			background: {BACKGROUND};
			color: {TEXT};
			border: 1px solid {BORDER};
			}

			QSpinBox::disabled {
				color: {TEXT_DISABLED};
			}

			QSpinBox::hover {
				background-color: {BACKGROUND};
				color: {TEXT_HOVER};
				border: 1px solid {BORDER};
			} 
			''',

		'QDoubleSpinBox': '''
			QDoubleSpinBox {
			background: {BACKGROUND};
			color: {TEXT};
			border: 1px solid {BORDER};
			}

			QDoubleSpinBox::disabled {
				color: {TEXT_DISABLED};
			}

			QDoubleSpinBox::hover {
				background-color: {BACKGROUND};
				color: {TEXT_HOVER};
				border: 1px solid {BORDER};
			} 
			''',

		'QAbstractSpinBox': '''
			QScrollBar:left-arrow, QScrollBar::right-arrow, QScrollBar::up-arrow, QScrollBar::down-arrow {
				border: 1px solid {PRESSED};
				width: 3px;
				height: 3px;
			}

			QAbstractSpinBox::up-arrow, QAbstractSpinBox::down-arrow {
				width: 3px;
				height: 3px;
				border: 1px solid {PRESSED};
			}

			QAbstractSpinBox::up-button, QAbstractSpinBox::down-button {
				border: 1px solid {PRESSED};
				background: {BACKGROUND};
				subcontrol-origin: border;
			} 
			''',

		'QCheckBox': '''
			QCheckBox {
				border-style: outset;
				border-radius: 1px;
				border: 1px solid {BORDER};
				padding: 0px 5px 0px 5px; /* top, right, bottom, left */
				background-color: {BACKGROUND};
				color: {TEXT};
				spacing: 5px;
			}

			QCheckBox::hover {
				background-color: {HOVER};
				color: {TEXT_HOVER};
			}

			QCheckBox::hover:checked {
				background-color: {HOVER};
				color: {TEXT_HOVER};
			}


			QCheckBox::enabled {
				color: {TEXT};
			}

			QCheckBox::disabled {
				color: {TEXT_DISABLED};
			}

			QCheckBox::disabled:checked {
				background-color: {BACKGROUND};
				color: {TEXT_DISABLED};
			}

			QCheckBox::checked {
				background-color: {HOVER};
				color: {TEXT_CHECKED};
			}

			QCheckBox::checked:hover {
				background-color: {HOVER};
				color: {TEXT_CHECKED};
			}

			QCheckBox::indeterminate {
				background-color: {HOVER};
				color: {TEXT_CHECKED};
			}

			QCheckBox::indeterminate:hover {
				background-color: {HOVER};
				color: {TEXT_CHECKED};
			}

			QCheckBox::indicator {
				width: 0px;
				height: 0px;
				border: 0px solid transparent;
			}

			QCheckBox::indicator::unchecked {
				image: none;
			}

			QCheckBox::indicator:unchecked:hover {
				image: none;
			}

			QCheckBox::indicator:unchecked:pressed {
				image: none;
			}

			QCheckBox::indicator::checked {
				image: none;
			}

			QCheckBox::indicator:checked:hover {
				image: none;
			} 

			QCheckBox::indicator:checked:pressed {
				image: none;
			}

			QCheckBox::indicator:indeterminate:checked {
				image: none;
			}

			QCheckBox::indicator:indeterminate:hover {
				image: none;
			}

			QCheckBox::indicator:indeterminate:pressed {
				image: none;
			}
			''',

		'QRadioButton': '''
			QRadioButton {
				border-style: outset;
				border-radius: 1px;
				border: 1px solid {BORDER};
				padding: 0px 5px 0px 5px; /* top, right, bottom, left */
				background-color: {BACKGROUND};
				color: {TEXT};
			}

			QRadioButton::hover {
				background-color: {HOVER};
				color: {TEXT_HOVER};
			}

			QRadioButton::hover:checked {
				background-color: {HOVER};
				color: {TEXT_HOVER};
			}

			QRadioButton::enabled {
				color: {TEXT};
			}

			QRadioButton::disabled {
				color: {TEXT_DISABLED};
			}

			QRadioButton::checked {
				background-color: {HOVER};
				color: {TEXT_CHECKED};
			}

			QRadioButton::checked:hover {
				background-color: {HOVER};
				color: {TEXT_CHECKED};
			}

			QRadioButton::indicator {
				width: 0px;
				height: 0px;
				border: 0px solid transparent;
			}

			QRadioButton::indicator::unchecked {
				image: none;
			}

			QRadioButton::indicator:unchecked:hover {
				image: none;
			}

			QRadioButton::indicator::checked {
				image: none;
			}

			QRadioButton::indicator:checked:hover {
				image: none;
			} 
			''',

		'QAbstractItemView': '''
			QAbstractItemView {
				show-decoration-selected: 1;
				selection-background-color: {HOVER};
				selection-color: {BACKGROUND};
				alternate-background-color: {BACKGROUND};
				min-width: 200px;
			} 
			''',

		'QHeaderView': '''
			QHeaderView {
				border: 1px solid {PRESSED};
			}

			QHeaderView::section {
				background: {PRESSED};
				border: 1px solid {PRESSED};
				padding: 1px;
			}

			QHeaderView::section:selected, QHeaderView::section::checked {
				background: {BACKGROUND};
			} 
			''',

		'QTableView': '''
			QTableView {
				gridline-color: {PRESSED};
			} 
			''',

		'QLineEdit': '''
			QLineEdit {
				border: 1px solid {BORDER};
				border-radius: 1px;
				padding: 0 8px;
				background: {BACKGROUND};
				color: {TEXT};
				selection-background-color: {HOVER};
				selection-color: {TEXT};
			}

			QLineEdit::disabled {
				color: {BACKGROUND};
			}

			QLineEdit::enabled {
				color: {TEXT};
			}

			QLineEdit:read-only {
				background: {BACKGROUND};
			} 
			''',

		'QTextEdit': '''
			QTextEdit#hud {
				border: 0px solid transparent;
				background-color: transparent;
				color: white;
				selection-background-color: {TEXT_BACKGROUND};
				selection-color: white;
			}

			QTextEdit {
				border: 1px solid {BORDER};
				background-color: {BACKGROUND};
				color: {TEXT};
				selection-background-color: {HOVER};
				selection-color: {TEXT};
				background-attachment: fixed; /* fixed, scroll */
			} 
			''',

		'QPlainTextEdit': '''
			QPlainTextEdit {
				
			} 
			''',

		'QListView': '''
			QListView {
				background-color: {BACKGROUND};
				color: {TEXT};
				alternate-background-color: {BACKGROUND};
				background-attachment: fixed; /* fixed, scroll */
			}

			QListView::item:alternate {
				background: {BACKGROUND};
			}

			QListView::item:selected {
				border: 1px solid {HOVER};
			}

			QListView::item:selected:!active {
				background: {HOVER};
				color: {TEXT};
			}

			QListView::item:selected:active {
				background: {HOVER};
				color: {TEXT};
			}

			QListView::item:hover {
				background: {HOVER};
				color: {TEXT_HOVER};
			} 
			''',

		'QListWidget': '''
			QListWidget {
				background-color: {BACKGROUND};
				color: {TEXT};
				alternate-background-color: {BACKGROUND};
				background-attachment: fixed; /* fixed, scroll */
			}

			QListWidget::item:alternate {
				background: {BACKGROUND};
			}

			QListWidget::item:selected {
				border: 1px solid {HOVER};
			}

			QListWidget::item:selected:!active {
				background: {HOVER};
				color: {TEXT};
			}

			QListWidget::item:selected:active {
				background: {HOVER};
				color: {TEXT};
			}

			QListWidget::item:hover {
				background: {HOVER};
				color: {TEXT_HOVER};
			} 
			''',

		'QTreeWidget': '''
			QTreeWidget {
				background-color: transparent;
				border:none;
			}

			QTreeWidget::item {
				height: 20px;
			}

			QTreeWidget::item:enabled {
				color: {TEXT};
			}

			QTreeWidget::item:disabled {
				color: {TEXT_DISABLED};
			}

			QTreeView::item:hover {
				background: {HOVER};
				color: {TEXT_HOVER};
			}

			QTreeView::item:selected {
				background-color: none;
			} 
			''',

		'QToolBox': '''
			QToolBox {
				background-color: {BACKGROUND};
				color: {TEXT};
				alternate-background-color: {BACKGROUND};
				background-attachment: fixed; /* fixed, scroll */
				icon-size: 0px;
			}

			QToolBox::QScrollArea:QWidget:QWidget {
				background: transparent;
			}

			QToolBox::QToolBoxButton {
				image: url(:/none);
			}
			
			QToolBox::QAbstractButton {
				background-image: url(:/none);
				image: url(:/none);
			}

			QToolBox::tab {
				background: {BACKGROUND};
				color: {TEXT};
				stop: 0.0 transparent, stop: 0.0 transparent,
				stop: 0.0 transparent, stop: 0.0 transparent;
				border-radius: 1px;
			}

			QToolBox::tab:selected {
				/*font: italic;*/ /* italicize selected tabs */
				color: {TEXT};
			} 
			''',

		'QAbstractSpinBox': '''
			QAbstractSpinBox {
				padding-right: 0px;
			} 
			''',

		'QSlider': '''
			QSlider {
				border: none;
			}

			QSlider::groove:horizontal {
				height: 5px;
				margin: 4px 0px 4px 0px; /* top, right, bottom, left */
			}

			QSlider::groove:vertical {
				width: 5px;
				margin: 0px 4px 0px 4px; /* top, right, bottom, left */
			}

			QSlider::handle {
				border: 1px solid {PRESSED};
				background: {BACKGROUND};
			}

			QSlider::handle:horizontal {
				width: 15px;
				margin: -4px 0px -4px 0px; /* top, right, bottom, left */
			}

			QSlider::handle:vertical {
				height: 15px;
				margin: 0px -4px 0px -4px; /* top, right, bottom, left */
			}

			QSlider::add-page:vertical, QSlider::sub-page:horizontal {
				background: {HOVER};
			}

			QSlider::sub-page:vertical, QSlider::add-page:horizontal {
				background: {BACKGROUND};
			} 
			''',

		'QScrollBar': '''
			QScrollBar {
				border: 1px solid transparent;
				background: {BACKGROUND};
			}

			QScrollBar:horizontal {
				height: 15px;
				margin: 0px 0px 0px 32px; /* top, right, bottom, left */
			}

			QScrollBar:vertical {
				width: 15px;
				margin: 32px 0px 0px 0px; /* top, right, bottom, left */
			}

			QScrollBar::handle {
				background: {PRESSED};
				border: 1px solid transparent;
			}

			QScrollBar::handle:horizontal {
				border-width: 0px 1px 0px 1px;
			}

			QScrollBar::handle:vertical {
				border-width: 1px 0px 1px 0px;
			}

			QScrollBar::handle:horizontal {
				min-width: 20px;
			}

			QScrollBar::handle:vertical {
				min-height: 20px;
			}

			QScrollBar::add-line, QScrollBar::sub-line {
				background:{BACKGROUND};
				border: 1px solid {PRESSED};
				subcontrol-origin: margin;
			}

			QScrollBar::add-line {
				position: absolute;
			}

			QScrollBar::add-line:horizontal {
				width: 15px;
				subcontrol-position: left;
				left: 15px;
			}

			QScrollBar::add-line:vertical {
				height: 15px;
				subcontrol-position: top;
				top: 15px;
			}

			QScrollBar::sub-line:horizontal {
				width: 15px;
				subcontrol-position: top left;
			}

			QScrollBar::sub-line:vertical {
				height: 15px;
				subcontrol-position: top;
			}

			QScrollBar::add-page, QScrollBar::sub-page {
				background: none;
			} 
			''',

		'QGroupBox': '''
			QGroupBox {
				border: 2px transparent;
				border-radius: 1px;
				margin: 10px 0px 0px 0px; /* top, right, bottom, left */ /* leave space at the top for the title */
				background-color: rgba(75,75,75,125);
			}

			QGroupBox::title {
				top: -12px;
				left: 2px;

				subcontrol-position: top left; /* position at the top center */
				background-color: transparent;
				color: {TEXT};
			} 
			''',

		'QTabBar': '''
			QTabBar {
				margin: 0px 0px 0px 2px; /* top, right, bottom, left */
			}

			QTabBar::tab {
				border-radius: 1px;
				padding-top: 1px;
				margin-top: 1px;
			}

			QTabBar::tab:selected {
				background: {BACKGROUND};
			} 
			''',

		'QMenu': '''
			QMenu {
				background-color: transparent;
				margin: 0px; /* spacing around the menu */
			}

			QMenu::item {
				padding: 8px 8px 8px 8px; /* top, right, bottom, left */
				border: 1px solid transparent; /* reserve space for selection border */
			}

			QMenu::item:selected {
				border-color: {HOVER};
				background: {BACKGROUND};
			}

			QMenu::icon:checked { /* appearance of a 'checked' icon */
				background: gray;
				border: 1px inset gray;
				position: absolute;
				top: 1px;
				right: 1px;
				bottom: 1px;
				left: 1px;
			}

			QMenu::separator {
				height: 2px;
				background: {BACKGROUND};
				margin: 0px 5px 0px 10px; /* top, right, bottom, left */
			}

			QMenu::indicator {
				width: 13px;
				height: 13px;
			} 
			''',

		'QMenuBar': '''
			QMenuBar {
				background-color: {BACKGROUND};
				spacing: 1px; /* spacing between menu bar items */
			}

			QMenuBar::item {
				padding: 1px 4px;
				background: transparent;
				border-radius: 1px;
			}

			QMenuBar::item:selected { /* when selected using mouse or keyboard */
				background: {HOVER};
			}

			QMenuBar::item:pressed {
				background: {TEXT_HOVER};
			}
			''',

		'QLabel': '''
			QLabel {
				background-color: {BACKGROUND};
				color: {TEXT};
				border: 1px solid {BORDER};
				border-radius: 1px;
				margin: 0px 0px 0px 0px; /* top, right, bottom, left */
				padding: 0px 5px 0px 5px; /* top, right, bottom, left */
			}

			QLabel::hover {   
				border: 1px solid {BORDER};
				background-color: {HOVER};
				color: {TEXT_HOVER};
			}

			QLabel::enabled {
				color: {TEXT};
			}

			QLabel::disabled {
				color: {TEXT_DISABLED};
			} 
			''',

		'QToolTip': '''
			QToolTip {
				background-color: {BACKGROUND};
				color: {TEXT};
				border: 0px solid transparent;
			} 
			''',

		'QProgressBar': '''
			QProgressBar {
				border: 0px solid black;
				border-radius: 5px;
				text-align: center;
				margin: 0px 0px 0px 0px; /* top, right, bottom, left */
			}

			QProgressBar::chunk {
				width: 1px;
				margin: 0px;
				background-color: {HOVER};
			} 
			''',

		'QFrame': '''
			QFrame {
				border: 0px solid black;
				border-radius: 1px;
				padding: 2px;
				background-image: none;
			} 
			''',

		'QSplitter': '''
			QSplitter::handle {
				image: url(images/splitter.png);
			}

			QSplitter::handle:horizontal {
				width: 2px;
			}

			QSplitter::handle:vertical {
				height: 2px;
			}

			QSplitter::handle:pressed {
				url(images/splitter_pressed.png);
			}
			''',

		'QSplitterHandle': '''
			QSplitter::handle:horizontal
			{
				border-left: 1px solid lightGray;
			}

			QSplitter::handle:vertical 
			{
				border-bottom: 1px solid lightGray;
			}
			''',

		'QTabWidget': '''
			QTabWidget {
				
			} 
			''',

		'QRubberBand': '''
			QRubberBand {
				color: 0px solid gray;
			}
			''',

		'QVBoxLayout': '''
			'QVBoxLayout' {

			}
			''',

		'QHBoxLayout': '''
			'QHBoxLayout' {

			}
			''',

		'QGridLayout': '''
			'QGridLayout' {

			}
			''',
		}

	@classmethod
	def getColorValues(cls, style='standard', **kwargs):
		'''Return the colorValues dict with any of the bracketed placeholders 
		replaced by the value of any given kwargs of the same name.

		:Parameters:
			style (str): The color value set to use. valid values are: 'standard', 'dark'
			**kwargs () = Keyword arguments matching the string of any bracketed placeholders.
				case insensitive.  ex. alpha=255

		:Return:
			(dict)
		'''
		kwargs = {k.upper():v for k, v in kwargs.items()} #assure upper case keys.

		colorValues_wAlpha = {
			k:v.format(**kwargs)
				for k, v in cls._colorValues[style].items()
		}

		return colorValues_wAlpha


	@classmethod
	def getStyleSheet(cls, widget_type=None, style='standard', backgroundOpacity=255):
		'''Get the styleSheet for the given widget type.
		By default it will return all stylesheets as one multi-line css string.

		:Parameters:
			widget_type (str): The class name of the widget. ie. 'QLabel'
			style (str): The color value set to use. valid values are: 'standard', 'dark'

		:Return:
			(str) css styleSheet
		'''
		if widget_type==None:
			return ''.join(cls._styleSheets.values())

		try:
			css = cls._styleSheets[widget_type]

		except KeyError as error:
			print ('{} in getStyleSheet\n\t# KeyError: {}: {} #'.format(__file__, widget_type, error))
			return ''

		for k, v in cls.getColorValues(style=style, alpha=backgroundOpacity).items():
			css = css.replace('{'+k.upper()+'}', v)

		return css


	@classmethod
	def setStyle(cls, widgets, ratio=6, style='standard', hideMenuButton=False, backgroundOpacity=255):
		'''Set the styleSheet for the given widgets.
		Set the style for a specific widget by using the '#' syntax and the widget's objectName. ie. QWidget#mainWindow

		:Parameters:
			widgets (obj)(list): A widget or list of widgets.
			ratio (int): The ratio of widget size, text length in relation to the amount of padding applied.
			style (str): Color mode. ie. 'standard' or 'dark'
			hideMenuButton (boool) = Hide the menu button of a widget that has one.
			backgroundOpacity (int): Set the background alpha transparency between 0 and 255.
		'''
		from uitk.switchboard import Switchboard

		widgets = [widgets] if not isinstance(widgets, (list, tuple, set, dict)) else widgets #assure widgets in a list.

		for widget in widgets:
			widget_type = Switchboard.getDerivedType(widget, name=True)

			try: #if hasattr(widget, 'styleSheet'):
				s = cls.getStyleSheet(widget_type, style=style, backgroundOpacity=backgroundOpacity)
				if hideMenuButton:
					s = s + cls.hideMenuButton(widget_type)

				try:
					length = len(widget.text()) if hasattr(widget, 'text') else len(str(widget.value())) #a 'NoneType' error will be thrown if the widget does not contain text.
					if widget.size().width() // length > ratio: #ratio of widget size, text length (using integer division).
						s = s + cls.adjustPadding(widget_type)
				except (AttributeError, ZeroDivisionError) as error:
					pass; # print (__name__, error, widget.text())

				if widget.styleSheet(): #if the widget has an existing style sheet, append.
					s = s+widget.styleSheet()
				widget.setStyleSheet(s)

			except AttributeError as error: #given widget has no attribute 'styleSheet'.
				pass; #print (error)


	@staticmethod
	def adjustPadding(widget_type):
		'''Remove padding when the text length / widget width ratio is below a given amount.
		'''
		return '''
			{0} {{
				padding: 0px 0px 0px 0px;
			}}'''.format(widget_type)


	@staticmethod
	def hideMenuButton(widget_type):
		'''Set the menu button as transparent.
		'''
		return '''
			{0}::menu-button {{
				border: 1px solid transparent;
			}}

			{0}::menu-button::hover {{
				border: 1px solid transparent;
			}}'''.format(widget_type)










if __name__ == "__main__":
	pass

# else:
# 	styleSheet = StyleSheet()

#module name
# print (__name__)
# --------------------------------------------------------------------------------------------
# Notes
# --------------------------------------------------------------------------------------------


'''
#qt styleSheet reference
http://doc.qt.io/qt-5/stylesheet-reference.html#qtabbar-widget



List of Pseudo-States:
:active				This state is set when the widget resides in an active window.
:adjoins-item		This state is set when the ::branch of a QTreeView is adjacent to an item.
:alternate			This state is set for every alternate row whe painting the row of a QAbstractItemView when QAbstractItemView::alternatingRowColors() is set to true.
:bottom				The item is positioned at the bottom. For example, a QTabBar that has its tabs positioned at the bottom.
:checked			The item is checked. For example, the checked state of QAbstractButton.
:closable			The items can be closed. For example, the QDockWidget has the QDockWidget::DockWidgetClosable feature turned on.
:closed				The item is in the closed state. For example, an non-expanded item in a QTreeView
:default			The item is the default. For example, a default QPushButton or a default action in a QMenu.
:disabled			The item is disabled.
:editable			The QComboBox is editable.
:edit-focus			The item has edit focus (See QStyle::State_HasEditFocus). This state is available only for Qt Extended applications.
:enabled			The item is enabled.
:exclusive			The item is part of an exclusive item group. For example, a menu item in a exclusive QActionGroup.
:first				The item is the first (in a list). For example, the first tab in a QTabBar.
:flat				The item is flat. For example, a flat QPushButton.
:floatable			The items can be floated. For example, the QDockWidget has the QDockWidget::DockWidgetFloatable feature turned on.
:focus				The item has input focus.
:has-children		The item has children. For example, an item in a QTreeView that has child items.
:has-siblings		The item has siblings. For example, an item in a QTreeView that siblings.
:horizontal			The item has horizontal orientation
:hover				The mouse is hovering over the item.
:indeterminate		The item has indeterminate state. For example, a QCheckBox or QRadioButton is partially checked.
:last				The item is the last (in a list). For example, the last tab in a QTabBar.
:left				The item is positioned at the left. For example, a QTabBar that has its tabs positioned at the left.
:maximized			The item is maximized. For example, a maximized QMdiSubWindow.
:middle				The item is in the middle (in a list). For example, a tab that is not in the beginning or the end in a QTabBar.
:minimized			The item is minimized. For example, a minimized QMdiSubWindow.
:movable			The item can be moved around. For example, the QDockWidget has the QDockWidget::DockWidgetMovable feature turned on.
:no-frame			The item has no frame. For example, a frameless QSpinBox or QLineEdit.
:non-exclusive		The item is part of a non-exclusive item group. For example, a menu item in a non-exclusive QActionGroup.
:off				For items that can be toggled, this applies to items in the "off" state.
:on					For items that can be toggled, this applies to widgets in the "on" state.
:only-one			The item is the only one (in a list). For example, a lone tab in a QTabBar.
:open				The item is in the open state. For example, an expanded item in a QTreeView, or a QComboBox or QPushButton with an open menu.
:next-selected		The next item (in a list) is selected. For example, the selected tab of a QTabBar is next to this item.
:pressed			The item is being pressed using the mouse.
:previous-selected	The previous item (in a list) is selected. For example, a tab in a QTabBar that is next to the selected tab.
:read-only			The item is marked read only or non-editable. For example, a read only QLineEdit or a non-editable QComboBox.
:right				The item is positioned at the right. For example, a QTabBar that has its tabs positioned at the right.
:selected			The item is selected. For example, the selected tab in a QTabBar or the selected item in a QMenu.
:top				The item is positioned at the top. For example, a QTabBar that has its tabs positioned at the top.
:unchecked			The item is unchecked.
:vertical			The item has vertical orientation.
:window				The widget is a window (i.e top level widget)


List of Sub-Controls:
::add-line			The button to add a line of a QScrollBar.
::add-page			The region between the handle (slider) and the add-line of a QScrollBar.
::branch			The branch indicator of a QTreeView.
::chunk				The progress chunk of a QProgressBar.
::close-button		The close button of a QDockWidget or tabs of QTabBar
::corner			The corner between two scrollbars in a QAbstractScrollArea
::down-arrow		The down arrow of a QComboBox, QHeaderView (sort indicator), QScrollBar or QSpinBox.
::down-button		The down button of a QScrollBar or a QSpinBox.
::drop-down			The drop-down button of a QComboBox.
::float-button		The float button of a QDockWidget
::groove			The groove of a QSlider.
::indicator			The indicator of a QAbstractItemView, a QCheckBox, a QRadioButton, a checkable QMenu item or a checkable QGroupBox.
::handle			The handle (slider) of a QScrollBar, a QSplitter, or a QSlider.
::icon				The icon of a QAbstractItemView or a QMenu.
::item				An item of a QAbstractItemView, a QMenuBar, a QMenu, or a QStatusBar.
::left-arrow		The left arrow of a QScrollBar.
::left-corner		The left corner of a QTabWidget. For example, this control can be used to control position the left corner widget in a QTabWidget.
::menu-arrow		The arrow of a QToolButton with a menu.
::menu-button		The menu button of a QToolButton.
::menu-indicator 	The menu indicator of a QPushButton.
::right-arrow		The right arrow of a QMenu or a QScrollBar.
::pane				The pane (frame) of a QTabWidget.
::right-corner		The right corner of a QTabWidget. For example, this control can be used to control the position the right corner widget in a QTabWidget.
::scroller			The scroller of a QMenu or QTabBar.
::section			The section of a QHeaderView.
::separator			The separator of a QMenu or in a QMainWindow.
::sub-line			The button to subtract a line of a QScrollBar.
::sub-page			The region between the handle (slider) and the sub-line of a QScrollBar.
::tab				The tab of a QTabBar or QToolBox.
::tab-bar			The tab bar of a QTabWidget. This subcontrol exists only to control the position of the QTabBar inside the QTabWidget. To style the tabs using the ::tab subcontrol.
::tear				The tear indicator of a QTabBar.
::tearoff			The tear-off indicator of a QMenu.
::text				The text of a QAbstractItemView.
::title				The title of a QGroupBox or a QDockWidget.
::up-arrow			The up arrow of a QHeaderView (sort indicator), QScrollBar or a QSpinBox.
::up-button			The up button of a QSpinBox.

List of Colors (Qt namespace (ie. Qt::red)):
white
black
red
darkRed
green
darkGreen
blue
darkBlue
cyan
darkCyan
magenta
darkMagenta
yellow
darkYellow
gray
darkGray
lightGray
color0 (zero pixel value) (transparent, i.e. background)
color1 (non-zero pixel value) (opaque, i.e. foreground)





image urls:
url(menu_indicator.png);
url(vline.png) 0;
url(handle.png);
url(close.png)
url(close-hover.png)
url(rightarrow.png);
url(leftarrow.png);
url(downarrow.png);
url(down_arrow.png);
url(up_arrow.png);
url(tear_indicator.png);
url(scrollbutton.png) 2;
url(branch-closed.png);
url(branch-open.png);
url(branch-more.png) 0;
url(branch-end.png) 0;
url(branch-open.png);
url(images/splitter.png);
url(images/splitter_pressed.png);
url(:/images/up_arrow.png);
url(:/images/up_arrow_disabled.png);
url(:/images/down_arrow.png);
url(:/images/down_arrow_disabled.png);
url(:/images/spindown.png) 1;
url(:/images/spindown_hover.png) 1;
url(:/images/spindown_pressed.png) 1;
url(:/images/sizegrip.png);
url(:/images/frame.png) 4;
url(:/images/spinup.png) 1;
url(:/images/spinup_hover.png) 1;
url(:/images/spinup_pressed.png) 1;
url(:/images/checkbox_unchecked.png);
url(:/images/checkbox_unchecked_hover.png);
url(:/images/checkbox_checked.png);
url(:/images/checkbox_checked_hover.png);
url(:/images/radiobutton_unchecked.png);
url(:/images/radiobutton_unchecked_hover.png);
url(:/images/radiobutton_checked.png);
url(:/images/radiobutton_checked_hover.png);
url(:/images/checkbox_indeterminate_hover.png);
url(:/images/checkbox_indeterminate_pressed.png);



'''

# min-width: 80px;

# QComboBox:editable {
# 	background: {BACKGROUND};
# }

# QComboBox:!editable, QComboBox::drop-down:editable {
# 	background: {BACKGROUND};
# }

# /* QComboBox gets the "on" state when the popup is open */
# QComboBox:!editable:on, QComboBox::drop-down:editable:on {
# 	background: {BACKGROUND};
# }
		
# background-color: #ABABAB; /* sets background of the menu */
# border: 1px solid black;

# /* sets background of menu item. set this to something non-transparent
# if you want menu color and menu item color to be different */
# background-color: transparent;

# QMenu::item:selected { /* when user selects item using mouse or keyboard */
#     background-color: #654321;
# For a more advanced customization, use a style sheet as follows:

# QMenu {
#     background-color: white;
#     margin: 2px; /* some spacing around the menu */

# QMenu::item {
#     padding: 2px 25px 2px 20px;
#     border: 1px solid transparent; /* reserve space for selection border */

# QMenu::item:selected {
#     border-color: darkblue;
#     background: rgba(100, 100, 100, 150);
# }

# QMenu::icon:checked { /* appearance of a 'checked' icon */
#     background: gray;
#     border: 1px inset gray;
#     position: absolute;
#     top: 1px;
#     right: 1px;
#     bottom: 1px;
#     left: 1px;

# QPushButton {
# 	background:rgba(127,127,127,2);
# 	background-color: red;
# 	color: white;
# 	border-width: 2px;
# 	border-radius: 10px;
# 	border-color: beige;
# 	border-style: outset;
# 	font: bold 14px;
# 	min-width: 10em;
# 	padding: 5px;

# QPushButton:hover {   
# 	border: 1px solid black;
# 	border-radius: 5px;   
# 	background-color:#66c0ff;

# QPushButton:pressed {
# 	background-color: rgb(224, 0, 0);
# 	border-style: inset;

# QPushButton:enabled {
# 	color: red

# QComboBox {
# 	image: url(:/none);}

# QComboBox::drop-down {
# 	border-width: 0px; 
# 	color: transparent

# QComboBox::down-arrow {
# 	border: 0px solid transparent; 
# 	border-width: 0px; left: 0px; top: 0px; width: 0px; height: 0px; 
# 	background-color: transparent; 
# 	color: transparent; 
# 	image: url(:/none);

# QTreeView {
#   alternate-background-color: rgba(35,35,35,255);
#   background: rgba(45,45,45,255);
# }

# QMenu {
# 	background-color: white; /* background-color: #ABABAB; sets background of the menu */
# 	margin: 2px; /* some spacing around the menu */
# 	border: 1px solid black;
# }

# QMenu::item {
# 	/* sets background of menu item. set this to something non-transparent
# 	if you want menu color and menu item color to be different */
# 	background-color: transparent;
# 	padding: 2px 25px 2px 20px;
# 	border: 1px solid transparent; /* reserve space for selection border */
# }

# QMenu::item:selected { 
# 	/* when user selects item using mouse or keyboard */
# 	background-color: #654321;
# 	border-color: darkblue;
# 	background: rgba(100, 100, 100, 150);
# }

# QMenu::icon:checked { /* appearance of a 'checked' icon */
# 	background: gray;
# 	border: 1px inset gray;
# 	position: absolute;
# 	top: 1px;
# 	right: 1px;
# 	bottom: 1px;
# 	left: 1px;
# }

# /* non-exclusive indicator = check box style indicator (see QActionGroup::setExclusive) */
# QMenu::indicator:non-exclusive:unchecked {
#     image: url(:/images/checkbox_unchecked.png);
# }

# QMenu::indicator:non-exclusive:unchecked:selected {
#     image: url(:/images/checkbox_unchecked_hover.png);
# }

# QMenu::indicator:non-exclusive:checked {
#     image: url(:/images/checkbox_checked.png);
# }

# QMenu::indicator:non-exclusive:checked:selected {
#     image: url(:/images/checkbox_checked_hover.png);
# }

# /* exclusive indicator = radio button style indicator (see QActionGroup::setExclusive) */
# QMenu::indicator:exclusive:unchecked {
#     image: url(:/images/radiobutton_unchecked.png);
# }

# QMenu::indicator:exclusive:unchecked:selected {
#     image: url(:/images/radiobutton_unchecked_hover.png);
# }

# QMenu::indicator:exclusive:checked {
#     image: url(:/images/radiobutton_checked.png);
# }

# QMenu::indicator:exclusive:checked:selected {
#     image: url(:/images/radiobutton_checked_hover.png);


# QToolButton:hover, QToolButton::menu-button:hover {
#     background: #787876;
# }

# QToolButton::checked{
#     background: #484846;
#     border: 1px solid #787876;
# }

# QToolButton:pressed, QToolButton::menu-button:pressed {
#     background: #787876;
# }

# QToolButton[popupMode="1"]{
# /* only for MenuButtonPopup */
#     padding-right: 30px;
#     background: red;
# }
# QToolButton[popupMode="2"]{
# /* only for OSC Server Status */
#     padding-right: 30px;
#     background: #484846;
# }
# QToolButton[popupMode="2"]:hover{
#     background: #787876;
# }
# QToolButton::down-arrow{
# } 
# /* the subcontrols below are used only in the MenuButtonPopup mode */
# QToolButton::menu-button{
# }

# QToolButton::menu-button:hover{
#     background: #787876;
# }
# QToolButton::menu-button:pressed{
# }
# QToolButton::menu-indicator{
#     bottom: 5px;
#     right: 5px;
# }


# !/usr/bin/python
# coding=utf-8
from uitk.slots import Slots



class Selection(Slots):
	'''
	'''
	def __init__(self, *args, **kwargs):
		super().__init__(*args, **kwargs)
		'''
		'''
		dh = self.sb.selection.draggable_header
		dh.ctxMenu.add(self.sb.ComboBox, setObjectName='cmb000', setToolTip='')
		dh.ctxMenu.add(self.sb.ComboBox, setObjectName='cmb006', setToolTip='A list of currently selected objects.')
		dh.ctxMenu.add('QCheckBox', setText='Ignore Backfacing', setObjectName='chk004', setToolTip='Ignore backfacing components during selection.')
		dh.ctxMenu.add('QCheckBox', setText='Soft Selection', setObjectName='chk008', setToolTip='Toggle soft selection mode.')
		dh.ctxMenu.add(self.sb.Label, setText='Grow Selection', setObjectName='lbl003', setToolTip='Grow the current selection.')
		dh.ctxMenu.add(self.sb.Label, setText='Shrink Selection', setObjectName='lbl004', setToolTip='Shrink the current selection.')
		dh.ctxMenu.cmb006.setCurrentText('Current Selection') # cmb.insertItem(cmb.currentIndex(), 'Current Selection') #insert item at current index.
		dh.ctxMenu.cmb006.popupStyle = 'qmenu'
		dh.ctxMenu.cmb006.beforePopupShown.connect(self.cmb006) #refresh the comboBox contents before showing it's popup.

		cmb001 = self.sb.selection.cmb001
		cmb001.ctxMenu.add(self.sb.Label, setText='Select', setObjectName='lbl005', setToolTip='Select the current set elements.')
		cmb001.ctxMenu.add(self.sb.Label, setText='New', setObjectName='lbl000', setToolTip='Create a new selection set.')
		cmb001.ctxMenu.add(self.sb.Label, setText='Modify', setObjectName='lbl001', setToolTip='Modify the current set by renaming and/or changing the selection.')
		cmb001.ctxMenu.add(self.sb.Label, setText='Delete', setObjectName='lbl002', setToolTip='Delete the current set.')
		cmb001.returnPressed.connect(lambda m=cmb001.ctxMenu.lastActiveChild: getattr(self, m(name=1))()) #connect to the last pressed child widget's corresponding method after return pressed. ie. self.lbl000 if cmb001.lbl000 was clicked last.
		cmb001.currentIndexChanged.connect(self.lbl005) #select current set on index change.
		cmb001.beforePopupShown.connect(self.cmb001) #refresh comboBox contents before showing it's popup.

		tb000 = self.sb.selection.tb000
		tb000.ctxMenu.add('QRadioButton', setText='Component Ring', setObjectName='chk000', setToolTip='Select component ring.')
		tb000.ctxMenu.add('QRadioButton', setText='Component Loop', setObjectName='chk001', setChecked=True, setToolTip='Select all contiguous components that form a loop with the current selection.')
		tb000.ctxMenu.add('QRadioButton', setText='Path Along Loop', setObjectName='chk009', setToolTip='The path along loop between two selected edges, vertices or UV\'s.')
		tb000.ctxMenu.add('QRadioButton', setText='Shortest Path', setObjectName='chk002', setToolTip='The shortest component path between two selected edges, vertices or UV\'s.')
		tb000.ctxMenu.add('QRadioButton', setText='Border Edges', setObjectName='chk010', setToolTip='Select the object(s) border edges.')
		tb000.ctxMenu.add('QSpinBox', setPrefix='Step: ', setObjectName='s003', setMinMax_='1-100 step1', setValue=1, setToolTip='Step Amount.')

		tb001 = self.sb.selection.tb001
		tb001.ctxMenu.add('QDoubleSpinBox', setPrefix='Tolerance: ', setObjectName='s000', setMinMax_='0.000-9999 step1.0', setValue=0.0, setToolTip='The allowed difference in any of the compared results.\nie. A tolerance of 4 allows for a difference of 4 components.\nie. A tolerance of 0.05 allows for that amount of variance between any of the bounding box values.')
		tb001.ctxMenu.add('QCheckBox', setText='Vertex', setObjectName='chk011', setChecked=True, setToolTip='The number of vertices.')
		tb001.ctxMenu.add('QCheckBox', setText='Edge', setObjectName='chk012', setChecked=True, setToolTip='The number of edges.')
		tb001.ctxMenu.add('QCheckBox', setText='Face', setObjectName='chk013', setChecked=True, setToolTip='The number of faces.')
		tb001.ctxMenu.add('QCheckBox', setText='Triangle', setObjectName='chk014', setToolTip='The number of triangles.')
		tb001.ctxMenu.add('QCheckBox', setText='Shell', setObjectName='chk015', setToolTip='The number of shells shells (disconnected pieces).')
		tb001.ctxMenu.add('QCheckBox', setText='Uv Coord', setObjectName='chk016', setToolTip='The number of uv coordinates (for the current map).')
		tb001.ctxMenu.add('QCheckBox', setText='Area', setObjectName='chk017', setToolTip='The surface area of the object\'s faces in local space.')
		tb001.ctxMenu.add('QCheckBox', setText='World Area', setObjectName='chk018', setChecked=True, setToolTip='The surface area of the object\'s faces in world space.')
		tb001.ctxMenu.add('QCheckBox', setText='Bounding Box', setObjectName='chk019', setToolTip='The object\'s bounding box in 3d space.\nCannot be used with the topological flags.')
		tb001.ctxMenu.add('QCheckBox', setText='Include Original', setObjectName='chk020', setToolTip='Include the original selected object(s) in the final selection.')
		tb001.ctxMenu.chk018.stateChanged.connect(lambda state: self.sb.toggleWidgets(tb001.ctxMenu, setDisabled='chk011-18') if state 
															else self.sb.toggleWidgets(tb001.ctxMenu, setEnabled='chk011-18')) #disable non-relevant options.
		tb002 = self.sb.selection.tb002
		tb002.ctxMenu.add('QCheckBox', setText='Lock Values', setObjectName='chk003', setChecked=True, setToolTip='Keep values in sync.')
		tb002.ctxMenu.add('QDoubleSpinBox', setPrefix='x: ', setObjectName='s002', setMinMax_='0.00-1 step.01', setValue=0.05, setToolTip='Normal X range.')
		tb002.ctxMenu.add('QDoubleSpinBox', setPrefix='y: ', setObjectName='s004', setMinMax_='0.00-1 step.01', setValue=0.05, setToolTip='Normal Y range.')
		tb002.ctxMenu.add('QDoubleSpinBox', setPrefix='z: ', setObjectName='s005', setMinMax_='0.00-1 step.01', setValue=0.05, setToolTip='Normal Z range.')

		tb003 = self.sb.selection.tb003
		tb003.ctxMenu.add('QDoubleSpinBox', setPrefix='Angle Low:  ', setObjectName='s006', setMinMax_='0.0-180 step1', setValue=70, setToolTip='Normal angle low range.')
		tb003.ctxMenu.add('QDoubleSpinBox', setPrefix='Angle High: ', setObjectName='s007', setMinMax_='0.0-180 step1', setValue=160, setToolTip='Normal angle high range.')


	def draggable_header(self, state=None):
		'''Context menu
		'''
		dh = self.sb.selection.draggable_header


	def s002(self, value=None):
		'''Select Island: tolerance x
		'''
		tb = self.sb.selection.tb002
		if tb.ctxMenu.chk003.isChecked():
			text = tb.ctxMenu.s002.value()
			tb.ctxMenu.s004.setValue(text)
			tb.ctxMenu.s005.setValue(text)


	def s004(self, value=None):
		'''Select Island: tolerance y
		'''
		tb = self.sb.selection.tb002
		if tb.ctxMenu.chk003.isChecked():
			text = tb.ctxMenu.s004.value()
			tb.ctxMenu.s002.setValue(text)
			tb.ctxMenu.s005.setValue(text)


	def s005(self, value=None):
		'''Select Island: tolerance z
		'''
		tb = self.sb.selection.tb002
		if tb.ctxMenu.chk003.isChecked():
			text = tb.ctxMenu.s005.value()
			tb.ctxMenu.s002.setValue(text)
			tb.ctxMenu.s004.setValue(text)


	def chk000(self, state=None):
		'''Select Nth: uncheck other checkboxes
		'''
		self.sb.toggleWidgets(setUnChecked='chk001-2')


	def chk001(self, state=None):
		'''Select Nth: uncheck other checkboxes
		'''
		self.sb.toggleWidgets(setUnChecked='chk000,chk002')


	def chk002(self, state=None):
		'''Select Nth: uncheck other checkboxes
		'''
		self.sb.toggleWidgets(setUnChecked='chk000-1')


	def chk005(self, state=None):
		'''Select Style: Marquee
		'''
		self.sb.toggleWidgets(setChecked='chk005', setUnChecked='chk006-7')
		Selection.setSelectionStyle('marquee')
		self.messageBox('Select Style: <hl>Marquee</hl>')


	def chk006(self, state=None):
		'''Select Style: Lasso
		'''
		self.sb.toggleWidgets(setChecked='chk006', setUnChecked='chk005,chk007')
		Selection.setSelectionStyle('lasso')
		self.messageBox('Select Style: <hl>Lasso</hl>')


	def chk007(self, state=None):
		'''Select Style: Paint
		'''
		self.sb.toggleWidgets(setChecked='chk007', setUnChecked='chk005-6')
		Selection.setSelectionStyle('paint')
		self.messageBox('Select Style: <hl>Paint</hl>')


	


# !/usr/bin/python
# coding=utf-8
from uitk.slots.maya import *
from uitk.slots.symmetry import Symmetry



class Symmetry_maya(Symmetry, Slots_maya):
	def __init__(self, *args, **kwargs):
		super().__init__(*args, **kwargs)

		cmb000 = self.sb.symmetry.draggable_header.ctxMenu.cmb000
		items = ['']
		cmb000.addItems_(items, '')

		#set initial checked state
		state = pm.symmetricModelling(query=True, symmetry=True) #application symmetry state
		axis = pm.symmetricModelling(query=True, axis=True)
		widget = 'chk000' if axis=='x' else 'chk001' if axis=='y' else 'chk002'
		getattr(self.sb.symmetry, widget).setChecked(state)
		getattr(self.sb.symmetry_submenu, widget).setChecked(state)


	def cmb000(self, index=-1):
		'''Editors
		'''
		cmb = self.sb.symmetry.draggable_header.ctxMenu.cmb000

		if index>0:
			if index==cmd.items.index(''):
				pass
			cmb.setCurrentIndex(0)


	def chk005(self, state=None):
		'''Symmetry: Topo
		'''
		self.sb.symmetry.chk004.setChecked(False) #uncheck symmetry:object space
		if any ([self.sb.symmetry.chk000.isChecked(), self.sb.symmetry.chk001.isChecked(), self.sb.symmetry.chk002.isChecked()]): #(symmetry)
			pm.symmetricModelling(edit=True, symmetry=False)
			self.sb.toggleWidgets(setUnChecked='chk000,chk001,chk002')
			self.messageBox('First select a seam edge and then check the symmetry button to enable topographic symmetry', messageType='Note')


	def setSymmetry(self, state, axis):
		space = "world" #workd space
		if self.sb.symmetry.chk004.isChecked(): #object space
			space = "object"
		elif self.sb.symmetry.chk005.isChecked(): #topological symmetry
			space = "topo"

		state = state if state==0 else 1 #for case when checkbox gives a state of 2.

		tolerance = 0.25
		pm.symmetricModelling(edit=True, symmetry=state, axis=axis, about=space, tolerance=tolerance)
		# if state:
		# 	self.messageBox('Symmetry: <hl>{}</hl>'.format(axis.upper(), messageType='Result'))









#module name
print (__name__)
# --------------------------------------------------------------------------------------------
# Notes
# --------------------------------------------------------------------------------------------


# !/usr/bin/python
# coding=utf-8
from PySide2 import QtCore, QtGui, QtWidgets

from attributes import Attributes
from text import RichText, TextOverlay
from menu import MenuInstance



class ComboBox(QtWidgets.QComboBox, MenuInstance, Attributes, RichText, TextOverlay):
	'''
	'''
	returnPressed = QtCore.Signal()
	beforePopupShown = QtCore.Signal()
	beforePopupHidden = QtCore.Signal()

	def __init__(self, parent=None, popupStyle='modelView', **kwargs):
		super().__init__(parent)
		'''
		:Parameters:
			popupStyle (str): specify the type of popup menu. default is the standard 'modelView'.
		'''
		self.popupStyle = popupStyle

		self.setStyleSheet(parent.styleSheet()) if parent else None

		# self.menu_.visible = False #built-in method isVisible() not working.
		self.view().installEventFilter(self)

		self.setAttributes(**kwargs)


	@property
	def items(self):
		'''Get a list of each items's text or it's data if it exists from the standard model/view.

		:Return:
			(list)
		'''
		return [self.itemData(i) if self.itemData(i) else self.itemText(i) for i in range(self.count())]


	def blockSignals_(fn):
		'''A decorator that blocks signals before executing a function, and unblocks them after.

		:Parameters:
			fn (obj): The function to be decorated.
		'''
		def wrapper(self, *args, **kwargs):
			self.blockSignals(True) #prevent triggering currentIndexChanged
			rtn = fn(self, *args, **kwargs)
			self.blockSignals(False)
			return rtn
		return wrapper


	@blockSignals_
	def addItems_(self, items, header=None, clear=True, ascending=False):
		'''Add items to the combobox's standard modelView without triggering any signals.

		:Parameters:
			items (str)(list)(dict): A string, list of strings, or dict with 'string':data pairs to fill the comboBox with.
			header (str): An optional value for the first index of the comboBox's list.
			clear (bool): Clear any previous items before adding new.
			ascending (bool): Insert in ascending order. New item(s) will be added to the top of the list.

		:Return:
			(list) comboBox's current item list minus any header.

		ex call: comboBox.addItems_(["Import file", "Import Options"], "Import")
		ex call: comboBox.addItems_({'Import file':<obj>, "Import Options":<obj>}, "Import") #example of adding items with data.
		'''
		assert isinstance(items, (str, list, set, tuple, dict)), '{}: addItems_: Incorrect datatype: {}'.format(__file__, type(items).__name__) 

		index = self.currentIndex() if self.currentIndex()>0 else 0 #get the current index before refreshing list. avoid negative values.

		if clear:
			self.clear()

		if isinstance(items, str):
			items = [items]
		if isinstance(items, (list, tuple, set)):
			items = {i:None for i in items}

		for item, data in items.items():
			if item is not None:
				if ascending:
					self.insertItem(0, item, data)
				else:
					self.addItem(item, data)

		if header:
			self.insertItem(-1, header)

		self.setCurrentIndex(index)

		return items


	@blockSignals_
	def currentData(self):
		'''Get the data at the current index.

		:Return:
			() data
		'''
		index = self.currentIndex()
		return self.itemData(index)


	@blockSignals_
	def setCurrentData(self, value):
		'''Sets the data for the current index.

		:Parameters:
			value () = The current item's data value.
		'''
		index = self.currentIndex()
		self.setItemData(index, value)


	@blockSignals_
	def currentText(self):
		'''Get the text at the current index.

		:Return:
			(str)
		'''
		index = self.currentIndex()
		return self.richText(index)


	@blockSignals_
	def setCurrentText(self, text):
		'''Sets the text for the current index.

		:Parameters:
			item (str): The current item's text value.
		'''
		index = self.currentIndex()
		self.setRichText(text, index)


	@blockSignals_
	def setItemText(self, index, text):
		'''Set the text at the given index.
		Override for setItemText built-in method.

		:Parameters:
			item (str): Item text.
			index (int): Item index
		'''
		self.setRichText(text, index)


	@blockSignals_
	def setCurrentItem(self, i):
		'''Sets the current item from the given item text or index without triggering any signals.

		:Parameters:
			i (str)(int): item text or item index
		'''
		try: #set by item index:
			self.setCurrentIndex(self.items.index(i))
		except Exception as error: #set by item text:
			try:
				self.setCurrentText(i)
			except Exception as error:
				if i:
					print ('{}: setCurrentItem: {}'.format(__file__, error))


	def showPopup(self):
		'''Show the popup menu.
		'''
		self.beforePopupShown.emit()

		if not self.popupStyle=='modelView':
			self.menu_.show() if not self.menu_.isVisible() else self.menu_.hide()
			return	

		else:
			width = self.sizeHint().width()
			self.view().setMinimumWidth(width)

		super().showPopup()


	def hidePopup(self):
		''''''
		self.beforePopupHidden.emit()

		if not self.popupStyle=='modelView':
			self.menu_.hide()
			# self.menu_.visible=False
		else:
			super().hidePopup()


	def clear(self):
		''''''
		if not self.popupStyle=='modelView':
			self.menu_.clear()
		else:
			super().clear()


	def enterEvent(self, event):
		'''
		:Parameters:
			event=<QEvent>
		'''

		super().enterEvent(event)


	def leaveEvent(self, event):
		'''
		:Parameters:
			event=<QEvent>
		'''
		# self.hidePopup()

		super().leaveEvent(event)


	def mousePressEvent(self, event):
		'''
		:Parameters:
			event=<QEvent>
		'''
		if event.button()==QtCore.Qt.RightButton:
			self.ctxMenu.show()

		super().mousePressEvent(event)


	def keyPressEvent(self, event):
		'''
		:Parameters:
			event = <QEvent>
		'''
		if event.key()==QtCore.Qt.Key_Return and not event.isAutoRepeat():
			self.returnPressed.emit()
			self.setEditable(False)

		super().keyPressEvent(event)


	def showEvent(self, event):
		'''
		:Parameters:
			event=<QEvent>
		'''
		if self.ctxMenu.containsMenuItems:
			# self.ctxMenu.setTitle(self.itemText(0))
			self.setTextOverlay('⧉', alignment='AlignRight')
			self.setItemText(0, self.itemText(0)) #set text: comboBox

		super().showEvent(event)


	def eventFilter(self, widget, event):
		'''Event filter for the standard view.

		QtCore.QEvent.Show, Hide, FocusIn, FocusOut, FocusAboutToChange
		'''
		# if not (str(event.type()).split('.')[-1]) in ['QPaintEvent', 'UpdateLater', 'PolishRequest', 'Paint']: print(str(event.type())) #debugging
		if event.type()==QtCore.QEvent.Hide:
			if self.parent().__class__.__name__=='Menu':
				self.parent().hide()

		return super().eventFilter(widget, event)









if __name__ == "__main__":
	import sys
	app = QtWidgets.QApplication.instance() or QtWidgets.QApplication(sys.argv) #return the existing QApplication object, or create a new one if none exists.

	w = ComboBox(popupStyle='qmenu')
	w.show()
	sys.exit(app.exec_())



# -----------------------------------------------------------------------------
# Notes
# -----------------------------------------------------------------------------

'''
Promoting a widget in designer to use a custom class:
>	In Qt Designer, select all the widgets you want to replace, 
		then right-click them and select 'Promote to...'. 

>	In the dialog:
		Base Class:		Class from which you inherit. ie. QWidget
		Promoted Class:	Name of the class. ie. "MyWidget"
		Header File:	Path of the file (changing the extension .py to .h)  ie. myfolder.mymodule.mywidget.h

>	Then click "Add", "Promote", 
		and you will see the class change from "QWidget" to "MyWidget" in the Object Inspector pane.
'''

# deprecated:




# !/usr/bin/python
# coding=utf-8
from uitk.slots import Slots



class Subdivision(Slots):
	'''
	'''
	def __init__(self, *args, **kwargs):
		super().__init__(*args, **kwargs)
		'''
		'''
	

	def draggable_header(self, state=None):
		'''Context menu
		'''
		dh = self.sb.subdivision.draggable_header

